"""All the public interfaces.

$Id: interfaces.py 127 2004-08-15 20:25:01Z bjorn $
"""

import zope.interface
import zope.schema

import fields

## LANGUAGE ##

class IReadLanguage(zope.interface.Interface):
    """Read interface for ILanguage."""

    def getWordCategory(label):
        """Returns the word category with the specified label."""

    def getInflectionFormGroup(label):
        """Returns the inflection form group with the given label."""

    def getInflectionForm(label):
        """Returns the inflection form with the given label."""

    id = zope.schema.Int(title=u'Unique ID',
                         description=
                         u"""Uniquely identifies the object.""",
                         readonly=True)

    label = zope.schema.TextLine(
        title=u'Name',
        description=u"""The name of the language.""",
        required=True)

    wordCategories = zope.schema.List(
        value_type=fields.WordCategory(),
        title=u'Word Categories',
        description=u"""A list of all word categories in the language.""")

    inflectionFormGroups = zope.schema.List(
        value_type=fields.InflectionFormGroup(),
        title=u'Inflection Form Groups',
        description=
        u"""A list of all inflection form groups in the language.""")

    inflectionForms = zope.schema.List(
        value_type=fields.InflectionForm(),
        title=u'Inflection Forms',
        description=u"""All defined inflection forms.""")


class IWriteLanguage(zope.interface.Interface):
    """Write interface for ILanguage."""

    def addWordCategory(label):
        """Adds a word category and returns it."""

    def addInflectionFormGroup(label, objectLabels):
        """Adds an inflection form group to the language.

        It should create the group, add objects to the group with the
        specified labels and return the added group.
        """

    def addWordList(label):
        """Adds a word list and returns it.""" 


class ILanguage(IReadLanguage, IWriteLanguage):
    """Language definition.

    It defines how a language is structured.
    """


## WORD ##

class IReadWordCategory(zope.interface.Interface):
    """Read interface for IWordCategory."""

    def getInflectionForm(*labels):
        """Gets the inflection form with the specified object labels."""

    def getInflectionForms(*labels):
        """Gets the inflection forms with the specified object labels."""

    def getExerciseDefinition(label):
        """Gets the exercise definition with the specified label."""
    
    def getQuizDefinition(label):
        """Gets the exercise definition with the specified label."""

    def getWord(value):
        """Returns a word.

        It matches the inflected word with the specified value and the
        word category's basic form as its inflection form, and returns
        its corresponding word object.
        """

    id = zope.schema.Int(title=u'Unique ID',
                         description=
                         u"""Uniquely identifies the object.""",
                         readonly=True)
        
    label = zope.schema.TextLine(
        title=u'Name',
        description=u"""The name of the word category.""",
        required=True)

    language = zope.schema.Object(
        schema=ILanguage,
        title=u'Language',
        description=u"""The language that the word category belongs to.""",
        required=True,
        readonly=True)

    inflectionForms = zope.schema.List(
        value_type=zope.schema.Choice(vocabulary="inflectionforms_language"),
        title=u'Inflection Forms',
        description=u"""A list of all inflection forms.""")

    basicForm  = zope.schema.Choice(
        title=u'Basic Form',
        description=
        u"""Indicates which inflection form that is the basic form of a word.

        An inflected word of this form should exist for every word.
        """,
        vocabulary="inflectionforms_self") 

    morphPropertyGroups = zope.schema.List(
        value_type=zope.schema.Choice(vocabulary=
                                      "inflectionformgroups_language"),
        title=u'Morphological Property Groups',
        description=u"""A list of all the morpholical property groups.""")

    morphProperties = zope.schema.List(
        title=u'Morphological Properties',
        description=u"""A list of all the morpholical properties.""",
        readonly=True)

    words = zope.schema.List(
        value_type=fields.Word(),
        title=u'Words',
        description=u"""All the words in the word category.""",
        readonly=False)

    exerciseDefinitions = zope.schema.List(
        value_type=fields.ExerciseDefinition(),
        title=u'Exercise Definitions',
        description=
        u"""All the exercise definitions defined in the word category.""",
        readonly=False)
    
    quizDefinitions = zope.schema.List(
        value_type=fields.QuizDefinition(),
        title=u'Quiz Definitions',
        description=
        u"""All the quiz definitions defined in the word category.""",
        readonly=False)


class IWriteWordCategory(zope.interface.Interface):
    """Write interface for IWordCategory."""

    def addInflectionForm(inflectionForm):
        """Adds a single inflection form."""

    def addInflectionForms(inflectionFormGroups):
        """Adds several inflection forms.

        It adds every inflection form that can be created from the
        inflection form groups.
        """

    def addMorphPropertyGroup(morpPropertyGroup):
        """Adds a morphological property group."""

    def addInflection(morphProperties,
                      inputForm, inputRegExps,
                      outputForms, outputRegExp):
        """Adds an inflection and returns it.

        It takes a list of morphological properties the inflection is
        valid for, the inflection form that is needed to produce the
        output, a list of input regular expressions, a list of
        inflection forms the inflection inflects to, and finally the
        output regular expression.
        """

    def addInflections(morphProperties,
                       inputForm, inputRegExps,
                       *outputMappings):
        """Adds several inflections at once.

        For every outputMapping an inflection is added with the
        specified morphological properties, input inflection form and
        input regular expressions.

        An output mapping is a tuple consisting of a list of output
        inflection forms and an output regular expression.
        """

    def addExerciseDefinition(label, inflectionForms):
        """Creates and adds an exercise.
        
        label is the name of the exercise, and inflectionForms is which
        inflection forms that can occur in the exercise.
        """
    
    def addQuizDefinition(label, exerciseDefinition, numberOfExercises):
        """Creates and adds a quiz."""
        
    def addWord(value, *morphProperties):
        """Adds a new word with the specified properties and returns it.

        It sets an inflected word of the basic inflection form with
        the value.
        """


class IWordCategory(IReadWordCategory, IWriteWordCategory):
    """Word Category definition.  """ 


class IReadWord(zope.interface.Interface):
    """Read interface for IWord."""

    label = zope.schema.TextLine(
        title=u'Name',
        description=u"""The visual representation of the word.

        Should be the inflected word of the basic form.""",
        readonly=True,                   
        required=True)
    
    wordCategory = zope.schema.Object(
        schema=IWordCategory,
        title=u'Word Category',
        description=u"""The word category the word belongs to.""",
        readonly=True,
        required=True)

    inflectedWords = zope.schema.List(
        title=u'Inflected Words',
        description=u"""A list of all the inflected forms of the word.""",
        readonly=True)

    morphProperties = zope.schema.List(
        value_type=zope.schema.Choice(vocabulary=
                                      "morphproperties_wordcategory"),
        title=u'Morphological Properties',
        description=
        u"""A list of all morphological properties the word defines.""")

    def __getitem__(inflectionForm):
        """Return the value of an inflected word.

        The return value should specify all the objects in the given
        inflection form.
        """

    def __unicode__():
        """Returns the basic inflected form of the word."""

    def __str__():
        """Returns the basic inflected form of the word."""
    

class IWriteWord(zope.interface.Interface):
    """Write interface for IWord."""

    def __setitem__(inflectionForm, value):
        """Sets the inflected word of the specified inflection form."""


class IWord(IReadWord, IWriteWord):
    """A word.

    It contains information about which word category it belongs to.
    It also has a number of word inflections.

    In Grammatista "walk" and "walks" are the same word, but they are
    different inflections. A word in Grammatista corresponds to the
    morphological term lexeme.
    """

class IWordList(zope.interface.Interface):
    """A collection of words.

    It's used to group together words into group, instead of using all
    words in the language.
    """
    
    language = zope.schema.Object(
        schema=ILanguage,
        title=u'Language',
        description=u"""The language, in which the list is defined.""",
        readonly=True,
        required=True)
    
    words = zope.schema.List(
        value_type=fields.Word(),
        title=u'Words',
        description=u"""All the words in the list.""",
        readonly=False)



## INFLECTION ##
    
class IReadInflectionFormGroup(zope.interface.Interface):
    """Read interface for IInflectionFormGroup"""

    def __getitem__(label):
        """Returns the object with the specified label."""
        
    label = zope.schema.TextLine(
        title=u'Name',
        description=u"""The name of the inflection form group.""",
        required=True)
    
    language = zope.schema.Object(
        schema=ILanguage,
        title=u'Language',
        description=u"""The language, in which the group is defined.""",
        readonly=True,
        required=True)
        
    objects = zope.schema.List(
        title=u'Objects',
        description=
        u"""A list of all inflection form objects in the group.""",
        readonly=True)

class IWriteInflectionFormGroup(zope.interface.Interface):
    """Write interface for IInflectionFormGroup"""

    def addObject(label):
        """Adds an inflection form object and returns it."""

class IInflectionFormGroup(IReadInflectionFormGroup,
                           IWriteInflectionFormGroup):
    """A group of inflection forms.

    Groups together several InflectionFormObjects. For example, we could
    group together all the different cases into one group. This makes
    it easy for different word categories to share information.
    """


class IInflectionFormObject(zope.interface.Interface):
    """This is a part of an inflection form.

    Take for an example the inflection form
    '1st person singular present tense'. It consists of three
    InflectionFormObjects, '1st person', 'singular', and
    'present tense'.
    """

    id = zope.schema.Int(
        title=u"Unique Identifier",
        description=u"""Uniquely identifies each object.""",
        readonly=True)

    label = zope.schema.TextLine(
        title=u'Name',
        description=u"""The name of the inflection form object.""",
        required=True)

    group = zope.schema.Object(
        schema=IInflectionFormGroup,
        title=u'Group',
        description=u"""The group the object belongs to.""",
        readonly=True,
        required=True)




class IReadInflectionForm(zope.interface.Interface):
    """Read interface for IInflectionForm."""

    id = zope.schema.Int(title=u'Unique ID',
                         description=
                         u"""Uniquely identifies the object.""",
                         readonly=True)
    
    label = zope.schema.TextLine(
        title=u'Name',
        description=u"""The name of the inflection form.""",
        required=True)

    language = zope.schema.Object(
        schema=ILanguage,
        title=u'Language',
        description=u"""The  belongs to.""",
        readonly=True,
        required=True)
    
    objects = zope.schema.List(
        value_type=zope.schema.Choice(vocabulary=
                                      "inflectionformobjects_language"),
        title=u'Objects',
        description=
        u"""The inflection form objects that make up the inflection form.""")


class IWriteInflectionForm(zope.interface.Interface):
    """Write interface for IInflectionForm."""

    def addObject(inflectionFormObject):
        """Adds an inflection form object to the inflection form.

        If there already exists an object with the same group as the
        added object, an AmbiguousInflectionFormException should be
        raised.
        """


class IInflectionForm(IReadInflectionForm, IWriteInflectionForm):
    """An inflection form.

    It consists of several inflection form objects, exactly one from
    each group the word category specifies.

    If several objects of the same groups are specified, a
    AmbiguousInflectionFormException should be raised.
    """


class IInflectedWord(zope.interface.Interface):
    """An inflected word.

    Even the basic form of the word is considered to be inflected. It
    specifies for which inflection form objects this inflection is
    valid.
    """

    word = zope.schema.Object(
        schema=IWord,
        title=u'Word',
        description=u"""The word that is inflected.""",
        readonly=True,
        required=True)

    value = zope.schema.TextLine(
        title=u'Value',
        description=u"""The actual inflection value of the word.""",
        required=True)

    inflectionForm = zope.schema.Object(
        schema=IInflectionForm,
        title=u'Inflection Form',
        description=u"""The inflection form of the inflected word.""",
        readonly=True,
        required=True)



class IInflection(zope.interface.Interface):
    """Specifies an inflection.

    That is, a transformation of a word from one inflection form to another.
    """

    wordCategory = zope.schema.Object(
        schema=IWordCategory,
        title=u'Word Category',
        description=u"""The word category the inflection applies to.""",
        readonly=True,
        required=True)

    inputInflectionForm = zope.schema.Choice(
        vocabulary='inflectionforms_wordcategory',
        title=u'Input Inflection Form',
        description=u"""The inflection form that is needed.""",
        required=True)

    morphProperties = zope.schema.List(
        value_type=zope.schema.Choice(vocabulary=
                                      'morphproperties_wordcategory'),
        title=u'Morphological Properties',
        description=
        u"A list of morphological properties this inflection is valid for.",
        required=True)
    
    inputRegExps = zope.schema.List(
        title=u'Input Regular Expressions',
        description=
        u"""A list of regular expressions for the input inflected word.

        It tries to match at least one regular expression to the input
        inflected word. It then uses this expression to do the
        inflection.

        If no match is found no inflection can be done.
        """,
        readonly=True)
        
    outputInflectionForms = zope.schema.List(
        value_type=zope.schema.Choice(vocabulary=
                                      "inflectionforms_wordcategory"),
        title=u'Output Inflection Forms',
        description=
        u"""A list of which inflection forms that will be provided.""",
        required=True)

    outputRegExp = zope.schema.TextLine(
        title=u'Output Regular Expression',
        description=
        u"""The regular expression which is used to form the output.

        This should be a string, not an RegExp instance.
        """,
        required=True)

    def inflect(word):
        """Inflects the word to the inflection form.

        If it's unable to inflect the word due to lack of input it
        returns None.
        """

    def addMorphProperty(morphProperty):
        """Adds a morphological property."""

    def addInputRegExp(regExp):
        """Adds an input regular expression."""

    def addOutputForm(inflectionForm):
        """Adds an inflection form the inflection provides."""


## EXERCISE ##
    
class IExercise(zope.interface.Interface):
    """An exercise.

    It gets an ExerciseDefition and creates the exercise according to
    it.
    """
    inflectionForm = zope.schema.Object(
        schema=IInflectionForm,
        title=u'Inflection Form',
        description=
        u"""The inflection form the wanted answer should be in.""",
        readonly=True,
        required=True)

    baseWord = zope.schema.TextLine(
        title=u"Base Word",
        description=
        u"""The word that should be inflected, in its basic form.""",
        readonly=True)

    correctAnswer = zope.schema.TextLine(
        title=u"Correct Answer",
        description=u"""The correct answer of the exercise.""",
        readonly=True)

    def checkAnswer(answer):
        """Returns True if the given answer is correct, otherwise False."""


class IQuizDefinition(zope.interface.Interface):
    """A Quiz definition.

    It defines how a quiz will look like. It specifies how the correction of
    the quiz should be displayed, and which types of exercises it contains.

    For now it only can contain one type of exercises.
    """
    id = zope.schema.Int(title=u'Unique ID',
                         description=
                         u"""Uniquely identifies the object.""",
                         readonly=True)

    label = zope.schema.TextLine(
        title=u"Name",
        description=u"""The name of the definition.""",
        required=True)
    
    wordCategory = zope.schema.Object(
        schema=IWordCategory,
        title=u'Word Category',
        description=u"""The word category that the quiz belongs to.""",
        required=True,
        readonly=True)

    exerciseDefinition = zope.schema.Choice(
        vocabulary='exercisedefinitions_wordcategory',
        title=u'Exercise Type',
        description=u"""The type of exercises contained in the quiz.""",
        required=True)

    numberOfExercises = zope.schema.Int(
        title=u'Number of Exercises',
        description=u"""The number of exercises contained in the quiz.""",
        required=True)


class IExerciseDefinition(zope.interface.Interface):
    """An exercise definition for a single word category.

    It defines which word category and which inflection forms to be
    used when an exercise is created.

    It chooses the words from the specified word lists. If no word list is
    specified, it chooses from every word in the whole system.
    """
    id = zope.schema.Int(title=u'Unique ID',
                         description=
                         u"""Uniquely identifies the object.""",
                         readonly=True)

    label = zope.schema.TextLine(
        title=u"Name",
        description=u"""The name of the definition.""",
        required=True)

    wordCategory = zope.schema.Object(
        schema=IWordCategory,
        title=u'Word Category',
        description=u"""The word category the exercise is for.""",
        readonly=True,
        required=True)

    wordLists = zope.schema.List(
        value_type=zope.schema.Choice(vocabulary=
                                      "wordlists_wordcategory"),
        title=u"Word Lists",
        description=u"""The word lists that the words get chosen from.""")

    inflectionForms = zope.schema.List(
        value_type=zope.schema.Choice(vocabulary=
                                      "inflectionforms_wordcategory"),
        title=u"Inflection Forms",
        description=
        u"""The inflection forms that the exercise can contain."""
        )

    def addInflectionForm(inflectionForm):
        """Adds an inflection form to the definition."""


## GRAMMATISTA ##

class IReadGrammatista(zope.interface.Interface):
    """Specifies all read operations."""

    def getLanguage(label):
        """Returns the language with the specified label.

        If no such language exists, None is returned.
        """
        
    languages = zope.schema.List(
        value_type=fields.Language(),
        title=u'Languages',
        description=
        u"""A list of all the languages defined in the system.""")
    

class IWriteGrammatista(zope.interface.Interface):
    """Specifies all write operations."""

    def addLanguage(label):
        """Adds a language with the specified label and returns it."""

    def delLanguage(name):
        """Deletes the language with the specified name from the system."""


class IGrammatista(IReadGrammatista, IWriteGrammatista):
    """The main application."""
