# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2004,2008 Bjorn Tillenius
#
# This file is part of Grammatista.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""Imports lithuanian into the database.

Usage: lithuanian.py [-t]

If -t is specified some sample words are added. Used for testing.
"""

import os
import sys

here = os.path.dirname(os.path.realpath(__file__))
sys.path.insert(0, os.path.join(here, '..', 'lib'))

from grammatista import Grammatista
from grammatista.database.store import get_store

def main(args=sys.argv[1:], stdout=sys.stdout, stderr=sys.stderr):
    gram = Grammatista()
    ## LANGUAGE ##
    print "Adding the language...",
    lang = gram.add_language(u"lithuanian", u"Lietuvių")
    print "done."

    ## INFLECTION FORM GROUPS ##
    print "Adding the inflection form groups...",
    case_names = [
        u'Nominative',
        u'Genitive',
        u'Dative',
        u'Accusative',
        u'Instrumental',
        u'Locative',
        u'Vocative',
        ]
    case = lang.add_inflection_form_group(u'case', u'Case')
    for case_name in case_names:
        case.add_object(case_name.lower(), case_name)
    numeral = lang.add_inflection_form_group(u'numeral', u'Numeral')
    for numeral_name in [u'Singular', u'Plural']:
        numeral.add_object(numeral_name.lower(), numeral_name)

    gender = lang.add_inflection_form_group(u'gender', u'Gender')
    for gender_name in [u'Masculine', u'Feminine']:
        gender.add_object(gender_name.lower(), gender_name)
    linksniuote = lang.add_inflection_form_group(
        u'linksniuote', u'Linksniuotė')
    linksniuote_names = [
        u'1 (-as,-is,-ys)',
        u'2 (-a,-ė)',
        u'3 (-is)',
        u'4 (-us)',
        u'5 (-uo, -ė)']
    for linksniuote_name in linksniuote_names:
        linksniuote.add_object(linksniuote_name.lower(), linksniuote_name)
    print "done."

    ## WORD CATEGORIES ##
    print "Adding the word categories...",
    noun = lang.add_word_category(u'noun', u'Noun')
    noun.add_inflection_forms(case, numeral)
    noun.add_morph_property_group(gender)
    noun.add_morph_property_group(linksniuote)
    print "done."


    ## INFLECTIONS ##

    print "Adding the inflections...",
    linksniuote_1 = linksniuote.get_object(u'1 (-as,-is,-ys)')
    masculine = gender.get_object(u'masculine')
    feminine = gender.get_object(u'feminine')
    nom_sing = noun.get_inflection_form(u'nominative', u'singular')
    loc_sing = noun.get_inflection_form(u'locative', u'singular')

    noun.add_inflections(
        morph_properties=(linksniuote_1, masculine),
        input_form=nom_sing,
        input_regexps=[u'(.*[^i])as$',],
        output_mappings=[
         (noun.get_inflection_forms(u'genitive', u'singular'), ur'\1o'),
         (noun.get_inflection_forms(u'dative', u'singular'), ur'\1ui'),
         (noun.get_inflection_forms(u'accusative', u'singular'), ur'\1ą'),
         (noun.get_inflection_forms(u'instrumental', u'singular'), ur'\1u'),
         (noun.get_inflection_forms(u'locative', u'singular'), ur'\1e'),
         (noun.get_inflection_forms(u'vocative', u'singular'), ur'\1e'),
         ])

    noun.add_inflections(
        (linksniuote_1, masculine), nom_sing, (u'(.*)ias$',),
        output_mappings=[
         (noun.get_inflection_forms(u'genitive', u'singular'), ur'\1io'),
         (noun.get_inflection_forms(u'dative', u'singular'), ur'\1iui'),
         (noun.get_inflection_forms(u'accusative', u'singular'), ur'\1ią'),
         (noun.get_inflection_forms(u'instrumental', u'singular'), ur'\1iu'),
         (noun.get_inflection_forms(u'locative', u'singular'), ur'\1yje'),
         (noun.get_inflection_forms(u'vocative', u'singular'), ur'\1i'),
         ])

    noun.add_inflections(
        (linksniuote_1, masculine), nom_sing, (u'(.*)is$',),
        output_mappings=[
         (noun.get_inflection_forms(u'genitive', u'singular'), ur'\1io'),
         (noun.get_inflection_forms(u'dative', u'singular'), ur'\1iui'),
         (noun.get_inflection_forms(u'accusative', u'singular'), ur'\1į'),
         (noun.get_inflection_forms(u'instrumental', u'singular'), ur'\1iu'),
         (noun.get_inflection_forms(u'locative', u'singular'), ur'\1yje'),
         (noun.get_inflection_forms(u'vocative', u'singular'), ur'\1i'),
         ])

    noun.add_inflections(
        (linksniuote_1, masculine), nom_sing, (u'(.*)ys$',),
        output_mappings=[
         (noun.get_inflection_forms(u'genitive', u'singular'), ur'\1io'),
         (noun.get_inflection_forms(u'dative', u'singular'), ur'\1iui'),
         (noun.get_inflection_forms(u'accusative', u'singular'), ur'\1į'),
         (noun.get_inflection_forms(u'instrumental', u'singular'), ur'\1iu'),
         (noun.get_inflection_forms(u'locative', u'singular'), ur'\1yje'),
         (noun.get_inflection_forms(u'vocative', u'singular'), ur'\1y'),
         ])

    noun.add_inflections(
        (linksniuote_1, masculine), nom_sing, (u'(.*)is$', u'(.*)ys$',),
        output_mappings=[
         (noun.get_inflection_forms(u'nominative', u'plural'), ur'\1iai'),
         (noun.get_inflection_forms(u'genitive', u'plural'), ur'\1ių'),
         (noun.get_inflection_forms(u'dative', u'plural'), ur'\1iams'),
         (noun.get_inflection_forms(u'accusative', u'plural'), ur'\1ius'),
         (noun.get_inflection_forms(u'instrumental', u'plural'), ur'\1iais'),
         (noun.get_inflection_forms(u'locative', u'plural'), ur'\1iuose'),
         ])

    noun.add_inflections(
        (linksniuote_1, masculine), nom_sing, (u'(.*)as$',),
        output_mappings=[
         (noun.get_inflection_forms(u'nominative', u'plural'), ur'\1ai'),
         (noun.get_inflection_forms(u'genitive', u'plural'), ur'\1ų'),
         (noun.get_inflection_forms(u'dative', u'plural'), ur'\1ams'),
         (noun.get_inflection_forms(u'accusative', u'plural'), ur'\1us'),
         (noun.get_inflection_forms(u'instrumental', u'plural'), ur'\1ais'),
         (noun.get_inflection_forms(u'locative', u'plural'), ur'\1uose'),
         ])

    # Vocative plural is always the same as nominative plural
    noun.add_inflection(
        morph_properties=list(linksniuote.objects) + list(gender.objects),
        input_form=noun.get_inflection_form(u'nominative', u'plural'),
        input_regexps=[u'(.*)'],
        output_forms=list(noun.get_inflection_forms(u'vocative', u'plural')),
        output_regexp=ur'\1')

    get_store().commit()
    print "Done."

    if '-t' in args:
        print "Adding some sample words...",
        namas = noun.add_word(u'namas', masculine, linksniuote_1)
        kelias = noun.add_word(u'kelias', masculine, linksniuote_1)
        brolis = noun.add_word(u'brolis', masculine, linksniuote_1)
        kambarys = noun.add_word(u'kambarys', masculine, linksniuote_1)
        print "Done."

        print "Adding some exercise definitions...",
        noun.add_exercise_definition(
            u'all-forms', u'All Forms', *noun.inflection_forms)
        noun.add_exercise_definition(
            u'only-nominative', u'Only Nominative',
            noun.get_inflection_forms(u'nominative'))
        noun.add_exercise_definition(
            u'only-singular', u'Only Singular',
            noun.get_inflection_forms(u'singular'))


if __name__ == "__main__":
    sys.exit(main())
