##############################################################################
#
# Copyright (c) 2008 Bjorn Tillenius
#
# This file is part of Grammatista.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from __future__ import with_statement
import os

from django.http import HttpResponse

from zope.pagetemplate.pagetemplatefile import PageTemplateFile


here = os.path.dirname(os.path.realpath(__file__))
template_dir = os.path.join(here, 'templates')
static_dir = os.path.join(here, 'static')

class GrammatistaTemplate(PageTemplateFile):
    """Template which makes it possible to pass variables to the template."""

    def __init__(self, template_name):
        super(GrammatistaTemplate, self).__init__(
            os.path.join(template_dir, template_name))

    def pt_getContext(self, args=(), options={}, **kw):
       rval = PageTemplateFile.pt_getContext(self, args=args)
       options.update(rval)
       return options


class BaseView(object):
    """Base view for all Grammatista Views."""

    template_name = None
    page_title = None
    template_variables = None

    def __init__(self, request, **template_variables):
        if self.template_name is None:
            raise AssertionError(
                "Subclasses must specify the template_name attribute.")
        if self.page_title is None:
            raise AssertionError(
                "Subclasses must specify the page_title attribute.")
        if not self.template_variables:
            self.template_variables = dict(page_title=self.page_title)
        self.template_variables.update(template_variables)
        self.request = request
        self.page_template = self.get_page_template()

    def get_page_template(self):
        """Return the GrammatistaTemplate instance to use."""
        return GrammatistaTemplate(self.template_name)

    def initialize(self):
        """Any custom initialization goes here."""
        pass

    def render(self):
        """Render the page.

        Return an HttpResponse.
        """
        main_template = GrammatistaTemplate('main-template.pt')
        return HttpResponse(self.page_template(
            main_template=main_template, **self.template_variables))

def render_static(request, file_name):
    """Render a static file."""
    # Only serve files in the static directory.
    file_name = file_name.rstrip('/')
    assert '..' not in file_name
    assert '/' not in file_name
    file_path = os.path.join(static_dir, file_name)
    with open(file_path) as static_file:
        content = static_file.read()
    return HttpResponse(content)


def render_view(view_class):
    """Render a view using the given view class.

    This function is meant to be used as a function passed to the Django
    url patterns.

    It returns a function that accepts a request. It will pass the
    request to the view class and render the page.
    """
    def render_view_with_request(request, **extra_params):
        view = view_class(request, **extra_params)
        view.initialize()
        return view.render()
    return render_view_with_request
