##############################################################################
#
# Copyright (c) 2008 Bjorn Tillenius
#
# This file is part of Grammatista.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import unittest

from django.http import HttpResponse

from grammatista.django.base import BaseView, render_view

class TestView(BaseView):
    page_title = 'need to be set'
    template_name = 'need to be set'
    def get_page_template(self):
        return None


class TestPageView(unittest.TestCase):

    def test_requires_template_name(self):
        # Subclasses need to set the page_template attribute.
        class NoPageTemplateView(BaseView):
            page_title = 'need to be set'
        self.assertRaises(AssertionError, NoPageTemplateView, None)

    def test_requires_page_title(self):
        # Subclasses need to set the page_title attribute.
        class NoPageTitleView(BaseView):
            template_name = 'need to be set'
            def get_page_template(self):
                return None
        self.assertRaises(AssertionError, NoPageTitleView, None)

    def test_page_title_in_template_variables(self):
        class PageTitleView(TestView):
            page_title = 'test page title'
        view = PageTitleView(None)
        self.assertEqual(
            view.template_variables['page_title'], 'test page title')

    def test_init_variables_in_template_variables(self):
        # Extra keyword parameters that get passed to __init__ are
        # stored in template_variables. This makes it possible to pass
        # variables from the url patters.
        view = TestView(None, foo='bar', bar='baz')
        self.assertTrue(view.template_variables['foo'], 'bar')
        self.assertTrue(view.template_variables['bar'], 'baz')

    def test_request_in_init(self):
        test_request = object()
        view = TestView(test_request)
        self.assertEqual(view.request, test_request)

    def test_page_template_in_init(self):
        # get_page_template() gets called in __init__, and the result
        # gets stored in the page_template attribute.
        test_page_template = object()
        class GetPageTemplateView(TestView):
            def get_page_template(self):
                return test_page_template
        view = GetPageTemplateView(None)
        self.assertEqual(view.page_template, test_page_template)

    def test_render(self):
        # render() calls self.page_template passing in the main_template
        # and all the variables in the template_variables attribute. It
        # wraps the result in an HttpResponse.
        class TestHttpResponse:
            def __init__(self, content):
                self.content = content
        class TestPageTemplate:
            def __init__(self, passed_in_keywords):
                self.passed_in_keywords = passed_in_keywords
            def __call__(self, **kw):
                self.passed_in_keywords.update(kw)
                return 'content'
        passed_in_keywords = {}
        class RenderView(TestView):
            template_variables = {'foo': 'bar'}
            def get_page_template(self):
                return TestPageTemplate(passed_in_keywords)
        from grammatista.django import base
        old_http_response = base.HttpResponse
        base.HttpResponse = TestHttpResponse
        view = RenderView(None)
        result = view.render()
        base.HttpResponse = old_http_response
        self.assertEqual(result.content, 'content')
        self.assertEqual(
            set(passed_in_keywords.keys()), set(['foo', 'main_template',]))


class TestRenderView(unittest.TestCase):

    def test_render_view(self):
        class RenderClass:
            initialize_called = False
            render_called = False
            def __init__(self, request):
                self.request = request
            def initialize(self):
                self.initialize_called = True
            def render(self):
                self.render_called = True
                return self
        render_function = render_view(RenderClass)
        test_request = object()
        result = render_function(test_request)
        self.assertTrue(result.initialize_called)
        self.assertTrue(result.render_called)
        self.assertTrue(result.request, test_request)

        # Extra parameters get passed to the view as well.
        class ExtraParameterClass(RenderClass):
            def __init__(self, request, **extra_params):
                self.request = request
                self.extra_params = extra_params
        render_function = render_view(ExtraParameterClass)
        result = render_function(None, foo='bar', bar='baz')
        self.assertEqual(result.extra_params, {'foo': 'bar', 'bar': 'baz'})

