##############################################################################
#
# Copyright (c) 2004,2008 Bjorn Tillenius
#
# This file is part of Grammatista.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""Exercise related classes and functions.

There are two main classes, ExerciseDefinition and
Exercise. ExerciseDefinition is the only one that is stored in the
database. A defintion can be adapted to IExercise in order to create
an Exercise.
"""

import random
from zope.interface import implements

from storm.base import Storm
from storm.properties import Int, Unicode
from storm.references import Reference, ReferenceSet
from storm.store import Store

from grammatista.interfaces import IExercise, IExerciseDefinition
from grammatista.interfaces import IQuizDefinition


class Exercise(object):
    """An adapter from IExerciseDefinition to IExercise."""
    implements(IExercise)

    def __init__(self, definition, word=None, inflection_form=None):
        store = Store.of(definition)
        if word is None:
            from grammatista.word import Word
            words = store.find(
                Word, Word.word_category == definition.word_category)
            row_nr = random.randint(0, words.count()-1)
            word = words[row_nr]
        self.base_word = word.get_inflected_value(
            definition.word_category.basic_form)
        self.word = word
        if inflection_form is None:
            self.inflection_form = random.choice(
                list(definition.inflection_forms))
        else:
            self.inflection_form = inflection_form
        self.correct_answer = word.get_inflected_value(self.inflection_form)
        assert self.correct_answer is not None

    def check_answer(self, answer):
        return answer == self.correct_answer


class ExerciseDefinition(Storm):
    """A definition of an exerices.

    The definition is used to create exercise objects.
    """
    implements(IExerciseDefinition)

    __storm_table__ = "ExerciseDefinition"

    id = Int(primary=True)
    string_id = Unicode()
    name = Unicode()
    word_category_id = Int(name='word_category')
    word_category = Reference(word_category_id, 'WordCategory.id')
    inflection_forms = ReferenceSet(
        id,
        'ExerciseDefinitionInflectionForm.exercise_definition_id',
        'ExerciseDefinitionInflectionForm.inflection_form_id',
        'InflectionForm.id')

    def __init__(self, string_id, name, word_category):
        self.string_id = string_id
        self.name = name
        self.word_category = word_category

    def add_inflection_form(self, inflection_form):
        self.inflection_forms.add(inflection_form)


class QuizDefinition(Storm):
    """See `IQuizDefinition`"""
    implements(IQuizDefinition)

    __storm_table__ = "QuizDefinition"

    id = Int(primary=True)
    string_id = Unicode()
    name = Unicode()
    word_category_id = Int(name='word_category')
    word_category = Reference(word_category_id, 'WordCategory.id')
    exercise_definition_id = Int(name='exercise_definition')
    exercise_definition = Reference(
        exercise_definition_id, 'ExerciseDefinition.id')
    number_of_exercises = Int()

    def __init__(self, string_id, name, word_category, exercise_definition,
                 number_of_exercises):
        self.string_id = string_id
        self.name = name
        self.word_category = word_category
        self.exercise_definition = exercise_definition
        self.number_of_exercises = number_of_exercises
