##############################################################################
#
# Copyright (c) 2004,2008 Bjorn Tillenius
#
# This file is part of Grammatista.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""Inflection related classes and functions."""

import re

import zope.interface

from storm.base import Storm
from storm.properties import Int, Unicode
from storm.references import Reference, ReferenceSet
from storm.store import Store

from grammatista.interfaces import (
    IInflectedWord,
    IInflection,
    IInflectionForm,
    IInflectionFormGroup,
    IInflectionFormObject,
    )
from grammatista.references import InflectionFormInflectionFormObject


def generate_combinations(lists):
    """Generates all possible combinations of one element from each list.

    Example:

        >>> generate_combinations([[1,2], [3,4]])
        [[1, 3], [1, 4], [2, 3], [2, 4]]
    """
    if len(lists) == 1:
        return [[element] for element in lists[0]]
    result = []
    for element in lists[0]:
        for tail in generate_combinations(lists[1:]):
            combination = [element]
            combination[1:] = tail
            result.append(combination)
    return result


def generate_inflection_forms(inflection_form_groups):
    """Generates all inflection forms that can be generated from the input.

    That is, it's given a list of inflection form groups. Then it
    generates all possible combinations of one object from each group.
    """
    language = inflection_form_groups[0].language
    list_of_objects = [group.objects for group in inflection_form_groups]
    combinations = generate_combinations(list_of_objects)
    for combination in combinations:
        inflection_form = InflectionForm(language=language)
        for object in combination:
            inflection_form.add_object(object)
        yield inflection_form


class AmbiguousInflectionFormException(Exception):
    """Too many objects to uniquely specify the inflection form."""


class InflectionFormGroup(Storm):
    """See IInflectionFormGroup."""

    zope.interface.implements(IInflectionFormGroup)

    __storm_table__ = "InflectionFormGroup"

    id = Int(primary=True)
    string_id = Unicode()
    name = Unicode()

    language_id = Int(name='language')
    language = Reference(language_id, 'Language.id')
    objects = ReferenceSet(id, 'InflectionFormObject.group_id')

    def __init__(self, string_id, name, language):
        self.string_id = string_id
        self.name = name
        self.language = language

    def add_object(self, string_id, name):
        """See IInflectionFormGroup."""
        store = Store.of(self)
        ob = store.add(
            InflectionFormObject(string_id=string_id, name=name, group=self))
        return ob

    def get_object(self, string_id):
        """See IInflectionFormGroup."""
        for object in self.objects:
            if object.string_id == string_id:
                return object


class InflectionFormObject(Storm):
    """See IInflectionFormObject"""

    zope.interface.implements(IInflectionFormObject)

    __storm_table__ = "InflectionFormObject"

    id = Int(primary=True)
    string_id = Unicode()
    name = Unicode()
    group_id = Int(name='group')
    group = Reference(group_id, 'InflectionFormGroup.id')

    def __init__(self, string_id, name, group):
        self.string_id = string_id
        self.name = name
        self.group = group


class InflectionForm(Storm):
    """An inflection form.

    An inflection is a collection of inflection form objects, where
    each object belongs to different group.
    """

    zope.interface.implements(IInflectionForm)

    __storm_table__ = "InflectionForm"

    @property
    def string_id(self):
        return ', '.join([ob.string_id for ob in self.objects])

    @property
    def name(self):
        return ', '.join([ob.name for ob in self.objects])

    id = Int(primary=True)
    language_id = Int(name='language')
    language = Reference(language_id, 'Language.id')

    objects = ReferenceSet(
        id,
        'InflectionFormInflectionFormObject.inflection_form_id',
        'InflectionFormInflectionFormObject.inflection_form_object_id',
        'InflectionFormObject.id',
        order_by=InflectionFormInflectionFormObject.inflection_form_object_id)

    def __init__(self, language):
        self.language = language

    def add_object(self, inflection_form_object):
        """See IInflectionForm"""
        for ob in self.objects:
            if ob.group == inflection_form_object.group:
                raise AmbiguousInflectionFormException
        self.objects.add(inflection_form_object)


class Inflection(Storm):
    """See IInflection."""

    zope.interface.implements(IInflection)

    __storm_table__ = "Inflection"

    id = Int(primary=True)
    word_category_id = Int(name='word_category')
    word_category = Reference(word_category_id, 'WordCategory.id')
    input_regexps = ReferenceSet(id, 'RegExp.inflection_id')
    input_inflection_form_id = Int(name='input_inflection_form')
    input_inflection_form = Reference(
        input_inflection_form_id, 'InflectionForm.id')
    output_regexp = Unicode()
    morph_properties = ReferenceSet(
        id,
        'InflectionMorphProperty.inflection_id',
        'InflectionMorphProperty.inflection_form_object_id',
        'InflectionFormObject.id')
    output_inflection_forms = ReferenceSet(
        id,
        'InflectionOutputInflectionForm.inflection_id',
        'InflectionOutputInflectionForm.inflection_form_id',
        'InflectionForm.id')

    def __init__(self, word_category, input_inflection_form, output_regexp):
        self.word_category = word_category
        self.input_inflection_form = input_inflection_form
        self.output_regexp = output_regexp

    def add_output_form(self, inflection_form):
        self.output_inflection_forms.add(inflection_form)

    def inflect(self, word):
        """See IInflection."""
        input_string = word.get_inflected_value(self.input_inflection_form)
        if input_string:
            for regexp in self.input_regexps:
                if regexp.match(input_string):
                    return regexp.sub(input_string)
        return None

    def add_input_regexp(self, regexp_string):
        """Adds an input regular expression.."""
        store = Store.of(self)
        return store.add(RegExp(value=regexp_string, inflection=self))

    def add_morph_property(self, morph_property):
        self.morph_properties.add(morph_property)


class RegExp(Storm):
    """A regular expression.

    It's needed since an Inflection can have many RegExps as input.
    """

    __storm_table__ = "RegExp"

    id = Int(primary=True)
    value = Unicode()

    inflection_id = Int(name='inflection')
    inflection = Reference(inflection_id, 'Inflection.id')
    _compiled_regexp = None

    def __init__(self, value, inflection):
        self.value = value
        self.inflection = inflection

    @property
    def compiled_regexp(self):
        if self._compiled_regexp is None:
            self._compiled_regexp = re.compile(self.value)
        return self._compiled_regexp

    def match(self, string):
        """Returns True if the string matches the regular expression."""
        return self.compiled_regexp.match(string) is not None

    def sub(self, string):
        """Substitues the regular expression in string and returns it.

        This method is only useful for input regular expressions. It
        gets the output expression from the inflection.
        """
        return self.compiled_regexp.sub(self.inflection.output_regexp, string)


class InflectedWord(Storm):
    """See IInflectedWord"""

    zope.interface.implements(IInflectedWord)

    __storm_table__ = "InflectedWord"

    id = Int(primary=True)
    word_id = Int(name="word")
    word = Reference(word_id, 'Word.id')
    value = Unicode()
    inflection_form_id = Int(name='inflection_form')
    inflection_form = Reference(inflection_form_id, 'InflectionForm.id')

    def __init__(self, word, inflection_form, value):
        self.word = word
        self.inflection_form = inflection_form
        self.value = value
