##############################################################################
#
# Copyright (c) 2004,2008 Bjorn Tillenius
#
# This file is part of Grammatista.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""All the interfaces."""

from zope.interface import Attribute, Interface
from zope.schema import Choice, Int, List, Object, TextLine

from grammatista import fields
from grammatista.fields import Name


class IHasStringIdAndName(Interface):
    """An object that has a string id  and a name."""
    string_id = Name(
        title=u"Id",
        description=(
            u"Short string id used to represent the object in a URL."),
        required=True)

    name = TextLine(
        title=u'Name',
        description=u"The name used to represent the object in normal text.",
        required=True)


class ILanguage(IHasStringIdAndName):
    """Language definition.

    It defines how the language is structured.

    XXX More information about the language definition.
    """

    def get_word_category(string_id):
        """Returns the word category by its.

        Returns None, if no such word category exists.
        """

    def get_inflection_form_group(string_id):
        """Returns the inflection form group with the given string_id."""

    id = Int(
        title=u'Unique ID',
        description=u"""Uniquely identifies the object. Shouldn't be used
                        by anything.""",
        readonly=True)

    word_categories = List(
        value_type=fields.WordCategory(),
        title=u'Word Categories',
        description=u"""A list of all word categories in the language.""")

    inflection_form_groups = List(
        value_type=fields.InflectionFormGroup(),
        title=u'Inflection Form Groups',
        description=
        u"""A list of all inflection form groups in the language.""")

    exercise_definitions = Attribute(
        "All the exercises defined in this language.")

    quiz_definitions = List(
        value_type=fields.QuizDefinition(),
        title=u"Quiz Definitions")

    def add_word_category(string_id, name):
        """Adds a word category and returns it."""

    def add_inflection_form_group(string_id, name):
        """Adds an inflection form group to the language.

        Create the group and add objects to the group with the specified
        string_id and name, and return the added group.
        """

    def add_word_list(string_id, name):
        """Adds a word list and returns it.""" 


## WORD ##

class IWordCategory(IHasStringIdAndName):
    """Word Category definition.

    XXX More information about the definition.
    """

    id = Int(title=u'Unique ID',
                         description=
                         u"""Uniquely identifies the object. Shouldn't be used
                         by anything.""",
                         readonly=True)

    language = Object(
        schema=ILanguage,
        title=u'Language',
        description=u"""The language that the word category belongs to.""",
        required=True,
        readonly=True)

    inflection_forms = List(
        value_type=Choice(vocabulary="inflectionforms_language"),
        title=u'Inflection Forms',
        description=u"""A list of all inflection forms.""")

    basic_form  = Choice(
        title=u'Basic Form',
        description=
        u"""Indicates which inflection form that is the basic form of a word.

        An inflected word of this form should exist for every word.
        """,
        vocabulary="inflectionforms_self") 

    morph_property_groups = List(
        value_type=Choice(vocabulary="inflectionformgroups_language"),
        title=u'Morphological Property Groups',
        description=u"""A list of all the morpholical property groups.""")

    morph_properties = List(
        title=u'Morphological Properties',
        description=u"""A list of all the morpholical properties.""",
        readonly=True)

    words = List(
        value_type=fields.Word(),
        title=u'Words',
        description=u"""All the words in the word category.""",
        readonly=False)

    exercise_definitions = List(
        value_type=fields.ExerciseDefinition(),
        title=u'Exercise Definitions',
        description=
        u"""All the exercise definitions defined in the word category.""",
        readonly=False)

    quiz_definitions = List(
        value_type=fields.QuizDefinition(),
        title=u'Quiz Definitions',
        description=
        u"""All the quiz definitions defined in the word category.""",
        readonly=False)


    def add_inflection_form(inflection_form):
        """Add a single inflection form."""

    def add_inflection_forms(*inflection_form_groups):
        """Adds several inflection forms.

        It adds every inflection form that can be created from the
        inflection form groups.
        """

    def add_morph_property_group(morph_property_group):
        """Adds a morphological property group."""

    def add_inflection(morph_properties,
                       input_form, input_regexps,
                       output_forms, output_regexp):
        """Add an inflection and returns it.

        It takes a list of morphological properties the inflection is
        valid for, the inflection form that is needed to produce the
        output, a list of input regular expressions, a list of
        inflection forms the inflection inflects to, and finally the
        output regular expression.
        """

    def add_inflections(morph_properties,
                       input_form, input_regexps,
                       output_mappings):
        """Adds several inflections at once.

        For every outputMapping an inflection is added with the
        specified morphological properties, input inflection form and
        input regular expressions.

        An output mapping is a tuple consisting of a list of output
        inflection forms and an output regular expression.
        """

    def add_exercise_definition(string_id, name, *inflection_forms):
        """Create and add an exercise with the specified name and forms."""

    #XXX: this should be on ILanguage instead.
    def add_quiz_definition(string_id, name, exercise_definition,
                            number_of_exercises):
        """Create and add a quiz."""

    def add_word(value, *morph_properties):
        """Add a new word with the specified properties and returns it.

        It sets an inflected word of the basic inflection form with
        the value.
        """

    def get_inflection_form(*string_ids):
        """Get the inflection form with the specified object names."""

    def get_inflection_forms(*string_ids):
        """Get the inflection forms with the specified object names."""

    def get_exercise_definition(string_id):
        """Gets the exercise definition with the specified string_id.

        Return None if no such exercise definition exists.
        """

    def get_quiz_definition(string_id):
        """Gets the exercise definition with the specified label.

        Return None if no such quiz definition exists.
        """

    def get_word(value):
        """Returns a word.

        It matches the inflected word with the specified value and the
        word category's basic form as its inflection form, and returns
        its corresponding word object.

        XXX Maybe it should be possible to specify an inflection form
        as well, but default to basic form?
        """


class IWord(IHasStringIdAndName):
    """A word.

    It contains information about which word category it belongs to.
    It also has a number of word inflections.

    In Grammatista "walk" and "walks" are the same word, but they are
    different inflections. A word in Grammatista corresponds to the
    morphological term lexeme.
    """

    word_category = Object(
        schema=IWordCategory,
        title=u'Word Category',
        description=u"""The word category the word belongs to.""",
        readonly=True,
        required=True)

    inflected_words = List(
        #value_type=Object(schema=IInflectedWord),
        title=u'Inflected Words',
        description=u"""A list of all the inflected forms of the word.""",
        readonly=True)

    morph_properties = List(
        value_type=Choice(
            vocabulary= "morphproperties_wordcategory"),
        title=u'Morphological Properties',
        description=
        u"""A list of all morphological properties the word defines.""")

    def add_morph_property(morph_property):
        """Add a morphological property."""

    def get_inflected_value(inflection_form):
        """Return the value of an inflected word.

        The return value should specify all the objects in the given
        inflection form.
        """

    def set_inflected_value(inflection_form, value):
        """Sets the inflected word of the specified inflection form.

        If value is None, the inflected word will be removed.
        """



class IWordList(IHasStringIdAndName):
    """A collection of words.

    It's used to group together words into group, instead of using all words in
    the language.
    """

    language = Object(
        schema=ILanguage,
        title=u'Language',
        description=u"""The language, in which the list is defined.""",
        readonly=True,
        required=True)

    words = List(
        value_type=fields.Word(),
        title=u'Words',
        description=u"""All the words in the list.""",
        readonly=False)

    def add_word(word):
        """Add a word to the word list."""



## INFLECTION ##

class IInflectionFormGroup(IHasStringIdAndName):
    """A group of inflection forms.

    Groups together several InflectionFormObjects. For example, we could
    group together all the different cases into one group. This makes
    it easy for different word categories to share information.
    """
    """Read interface for IInflectionFormGroup"""

    language = Object(
        schema=ILanguage,
        title=u'Language',
        description=u"""The language, in which the group is defined.""",
        readonly=True,
        required=True)

    objects = List(
        #value_type=Object(schema=IInflectionFormObject),
        title=u'Objects',
        description=u"""A list of all inflection form objects in the group.""",
        readonly=True)

    def add_object(string_id, name):
        """Adds an inflection form object and returns it."""

    def get_object(string_id):
        """Returns the object with the specified string_id."""


class IInflectionFormObject(IHasStringIdAndName):
    """This is a part of an inflection form.

    Take for an example the inflection form '1st person singular present tense'
    It consists of three InflectionFormObjects, '1st person', 'singular',
    and 'present tense'.
    """

    id = Int(
        title=u"Unique Identifier",
        description=u"""Uniquely identifies each object.""",
        readonly=True)

    group = Object(
        schema=IInflectionFormGroup,
        title=u'Group',
        description=u"""The group the object belongs to.""",
        readonly=True,
        required=True)


class IInflectionForm(IHasStringIdAndName):
    """An inflection form.

    It consists of several inflection form objects, exactly one from
    each group the word category specifies.

    If several objects of the same groups are specified, a
    AmbiguousInflectionFormException should be raised.
    """
    id = Int(title=u'Unique ID',
                         description=
                         u"""Uniquely identifies the object.""",
                         readonly=True)

    language = Object(
        schema=ILanguage,
        title=u'Language',
        description=u"""The language the inflection form belongs to.""",
        readonly=True,
        required=True)

    objects = List(
        value_type=Choice(vocabulary="inflectionformobjects_language"),
        title=u'Objects',
        description=
        u"""The inflection form objects that make up the inflection form.""")


    def add_object(inflection_form_object):
        """Adds an inflection form object to the inflection form.

        If there already exists an object with the same group as the
        added object, an AmbiguousInflectionFormException should be
        raised.
        """


class IInflectedWord(Interface):
    """An inflected word.

    Even the basic form of the word is considered to be inflected. It
    specifies for which inflection form objects this inflection is
    valid.
    """

    word = Object(
        schema=IWord,
        title=u'Word',
        description=u"""The word that is inflected.""",
        readonly=True,
        required=True)

    value = TextLine(
        title=u'Value',
        description=u"""The actual inflection value of the word.""",
        required=True)

    inflection_form = Object(
        schema=IInflectionForm,
        title=u'Inflection Form',
        description=u"""The inflection form of the inflected word.""",
        readonly=True,
        required=True)



class IInflection(Interface):
    """Specifies an inflection.

    That is, a transformation of a word from one inflection form to another.

    XXX an example would probably be nice.
    """

    word_category = Object(
        schema=IWordCategory,
        title=u'Word Category',
        description=u"""The word category the inflection applies to.""",
        readonly=True,
        required=True)

    input_inflection_form = Choice(
        vocabulary='inflectionforms_wordcategory',
        title=u'Input Inflection Form',
        description=u"""The inflection form that is needed.""",
        required=True)

    #XXX Maybe abstract this the same way as with inflection forms?
    morph_properties = List(
        value_type=Choice(vocabulary='morphproperties_wordcategory'),
        title=u'Morphological Properties',
        description=
        u"A list of morphological properties this inflection is valid for.",
        required=True)

    input_regexps = List(
        #value_type=Object(schema=IRegExp),
        title=u'Input Regular Expressions',
        description=
        u"""A list of regular expressions for the input inflected word.

        XXX This should go into the class' doc string!
        It tries to match at least one regular expression to the input
        inflected word. It then uses this expression to do the
        inflection.

        If no match is found no inflection can be done.
        """,
        readonly=True)

    output_inflection_forms = List(
        value_type=Choice(vocabulary=
                                      "inflectionforms_wordcategory"),
        title=u'Output Inflection Forms',
        description=
        u"""A list of which inflection form objects that will be provided.""",
        required=True)

    output_regexp = TextLine(
        title=u'Output Regular Expression',
        description=
        u"""The regular expression which is used to form the output.

        This is a string, not an RegExp instance.
        """,
        required=True)

    def inflect(word):
        """Inflects the word to the inflection form.

        If it's unable to inflect the word due to lack of input it
        returns None.
        """

    def add_morph_property(morph_property):
        """Adds a morphological property."""

    def add_input_regexp(regexp):
        """Adds an input regular expression."""

    def add_output_form(inflection_form):
        """Adds an inflection form the inflection provides."""


## EXERCISE ##

class IExercise(Interface):
    """An exercise.

    It gets an ExerciseDefition and creates the exercise according to
    it.
    """
    inflection_form = Object(
        schema=IInflectionForm,
        title=u'Inflection Form',
        description=u"""The inflection form the wanted answer should be in.""",
        readonly=True,
        required=True)

    base_word = TextLine(
        title=u"Base Word",
        description=
        u"""The word that should be inflected, in its basic form.""",
        readonly=True)

    correct_answer = TextLine(
        title=u"Correct Answer",
        description=u"""The correct answer of the exercise.""",
        readonly=True)

    def check_answer(answer):
        """Return True if the given answer is correct, otherwise False."""


class IQuizDefinition(IHasStringIdAndName):
    """A Quiz definition.

    It defines how a quiz will look like. It specifies how the correction of
    the quiz should be displayed, and which types of exercises it contains.

    For now it only can contain one type of exercises.
    """
    id = Int(title=u'Unique ID',
                         description=
                         u"""Uniquely identifies the object.""",
                         readonly=True)

    word_category = Object(
        schema=IWordCategory,
        title=u'Word Category',
        description=u"""The word category that the quiz belongs to.""",
        required=True,
        readonly=True)

    exercise_definition = Choice(
        vocabulary='exercisedefinitions_wordcategory',
        title=u'Exercise Type',
        description=u"""The type of exercises contained in the quiz.""",
        required=True)

    number_of_exercises = Int(
        title=u'Number of Exercises',
        description=u"""The number of exercises contained in the quiz.""",
        required=True)


class IExerciseDefinition(IHasStringIdAndName):
    """An exercise definition for a single word category.

    It defines which word category and which inflection forms to be
    used when an exercise is created.

    It chooses the words from the specified word lists. If no word list is
    specified, it chooses from every word in the whole system.
    """
    id = Int(title=u'Unique ID',
                         description=
                         u"""Uniquely identifies the object.""",
                         readonly=True)

    word_category = Object(
        schema=IWordCategory,
        title=u'Word Category',
        description=u"""The word category the exercise is for.""",
        readonly=True,
        required=True)

    inflection_forms = List(
        value_type=Choice(vocabulary=
                                      "inflectionforms_wordcategory"),
        title=u"Inflection Forms",
        description=u"""The inflection forms which the exercise can contain."""
        )

    def add_inflection_form(inflection_form):
        """Adds an inflection form to the definition."""


class IGrammatista(Interface):
    """The main application."""

    def get_language(string_id):
        """Returns the language with the specified string_id.

        If no such language exists, None is returned.
        """

    def add_language(string_id, name):
        """Adds a language with the specified string_id and name.

        The newly created language is returned.
        """

    def remove_language(string_id):
        """Delete the language with the specified string id."""

    languages = List(
        value_type=fields.Language(),
        title=u'Languages',
        description=u"""A list of all the languages defined in the system.""")
