##############################################################################
#
# Copyright (c) 2004,2008 Bjorn Tillenius
#
# This file is part of Grammatista.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""Language related classes and functions."""

from zope.interface import implements

from storm.base import Storm
from storm.properties import Int, Unicode
from storm.references import ReferenceSet
from storm.store import Store

from grammatista.inflection import InflectionFormGroup
from grammatista.interfaces import ILanguage
from grammatista.word import WordCategory, WordList


class Language(Storm):
    """An implementation of ILanguage."""

    implements(ILanguage)

    __storm_table__ = "language"

    id = Int(primary=True)
    string_id = Unicode()
    name = Unicode()
    word_categories = ReferenceSet(id, 'WordCategory.language_id')
    word_lists = ReferenceSet(id, 'WordList.language_id')
    inflection_form_groups = ReferenceSet(
        id, 'InflectionFormGroup.language_id')
    inflection_forms = ReferenceSet(
        id, 'InflectionForm.language_id')

    def __init__(self, string_id, name):
        self.string_id = string_id
        self.name = name

    @property
    def exercise_definitions(self):
        exercise_definitions = set()
        for word_category in self.word_categories:
            exercise_definitions.update(word_category.exercise_definitions)
        return exercise_definitions

    @property
    def quiz_definitions(self):
        quiz_definitions = set()
        for word_category in self.word_categories:
            quiz_definitions.update(word_category.quiz_definitions)
        return quiz_definitions

    def add_word_list(self, string_id, name):
        """Adds a word list and returns it."""
        store = Store.of(self)
        return store.add(
            WordList(string_id=string_id, name=name, language=self))

    def add_word_category(self, string_id, name):
        """See ILanguage."""
        store = Store.of(self)
        return store.add(
            WordCategory(string_id=string_id, name=name, language=self))

    def get_word_category(self, string_id):
        """See ILanguage."""
        store = Store.of(self)
        results = store.find(
            WordCategory,
            WordCategory.language == self,
            WordCategory.string_id == string_id)
        return results.one()

    def get_inflection_form_group(self, string_id):
        """See ILanguage."""
        store = Store.of(self)
        results = store.find(
            InflectionFormGroup,
            InflectionFormGroup.language == self,
            InflectionFormGroup.string_id == string_id)
        return results.one()

    def add_inflection_form_group(self, string_id, name):
        """See ILanguage."""
        group = InflectionFormGroup(
            string_id=string_id, name=name, language=self)
        store = Store.of(self)
        inflection_form_group = store.add(group)
        store.flush()
        return inflection_form_group
