##############################################################################
#
# Copyright (c) 2008 Bjorn Tillenius
#
# This file is part of Grammatista.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""Factory for creating objects in tests."""

import uuid

from grammatista import Grammatista
from grammatista.exercise import Exercise
from grammatista.inflection import Inflection, InflectionForm


class ObjectFactory(object):

    def __init__(self):
        self.grammatista = Grammatista()

    def reset_db(self):
        """Recreate an empty DB."""

    def get_unique_string(self):
        """Returns a randomly generated unique string."""
        return unicode(uuid.uuid4())

    def make_language(self, string_id=None, name=None):
        if string_id is None:
            string_id = self.get_unique_string()
        if name is None:
            name = self.get_unique_string()
        return self.grammatista.add_language(
            string_id=string_id, name=name)

    def make_word_category(self, string_id=None, language=None):
        if string_id is None:
            string_id = self.get_unique_string()
        if language is None:
            language = self.make_language()
        return language.add_word_category(string_id, string_id)

    def make_word_list(self, string_id=None, language=None):
        if string_id is None:
            string_id = self.get_unique_string()
        if language is None:
            language = self.make_language()
        return language.add_word_list(string_id=string_id, name=string_id)

    def make_inflection_form_group(self, string_id=None, language=None,
                                   object_string_ids=None):
        if string_id is None:
            string_id = self.get_unique_string()
        if language is None:
            language = self.make_language()
        if object_string_ids is None:
            object_string_ids = []
        form_group = language.add_inflection_form_group(string_id, string_id)
        for object_string_id in object_string_ids:
            form_group.add_object(object_string_id, object_string_id.upper())
        return form_group

    def make_inflection_form_object(self, string_id=None, group=None):
        if string_id is None:
            string_id = self.get_unique_string()
        if group is None:
            group = self.make_inflection_form_group()
        return group.add_object(string_id, string_id)

    def make_inflection_form(self, language=None):
        if language is None:
            language = self.make_language()
        return InflectionForm(language=language)

    def make_inflection(self, inflection_form=None, output_regexp=None,
                        word_category=None):
        if inflection_form is None:
            inflection_form = self.make_inflection_form()
        if output_regexp is None:
            output_regexp = self.get_unique_string()
        if word_category is None:
            word_category = self.make_word_category()
        return Inflection(
            input_inflection_form=inflection_form,
            output_regexp=output_regexp, word_category=word_category)

    def make_regexp(self, value=None, inflection=None):
        if value is None:
            value = self.get_unique_string()
        if inflection is None:
            inflection = self.make_inflection()
        return inflection.add_input_regexp(value)

    def make_word(self, value=None, word_category=None):
        if value is None:
            value = self.get_unique_string()
        if word_category is None:
            word_category = self.make_word_category()

        inflection_group = self.make_inflection_form_group(
            language=word_category.language,
            object_string_ids=[self.get_unique_string()])
        morph_string_id = self.get_unique_string()
        morph_group = self.make_inflection_form_group(
            language=word_category.language,
            object_string_ids=[morph_string_id])
        word_category.add_inflection_forms(inflection_group)
        word_category.add_morph_property_group(morph_group)
        return word_category.add_word(
            value, morph_group.get_object(morph_string_id))

    def make_exercise_definition(self, string_id=None, word_category=None):
        if string_id is None:
            string_id = self.get_unique_string()
        if word_category is None:
            word_category = self.make_word_category()
        return word_category.add_exercise_definition(string_id, string_id)

    def make_quiz_definition(self, string_id=None, exercise_definition=None):
        if string_id is None:
            string_id = self.get_unique_string()
        if exercise_definition is None:
            exercise_definition = self.make_exercise_definition()
        word_category = exercise_definition.word_category
        return word_category.add_quiz_definition(
            string_id, string_id, exercise_definition, 1)

    def make_exercise(self, exercise_definition=None, word=None,
                      exercise_inflection_form=None, answer=None):
        word_category = self.make_word_category()
        if exercise_definition is None:
            exercise_definition = self.make_exercise_definition(
                word_category=word_category)
        basic_inflection_form = self.make_inflection_form(
            language=word_category.language)
        if exercise_inflection_form is None:
            exercise_inflection_form = self.make_inflection_form(
                language=word_category.language)
        word_category.add_inflection_form(basic_inflection_form)
        word_category.add_inflection_form(exercise_inflection_form)
        if word is None:
            word = self.make_word(word_category=word_category)
        if answer is None:
            answer = self.get_unique_string()
        word.set_inflected_value(
            basic_inflection_form, self.get_unique_string())
        word.set_inflected_value(exercise_inflection_form, answer)
        exercise_definition.add_inflection_form(exercise_inflection_form)
        return Exercise(exercise_definition, word, exercise_inflection_form)
