##############################################################################
#
# Copyright (c) 2008 Bjorn Tillenius
#
# This file is part of Grammatista.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""Tests for exercises and related things."""

import unittest

from zope.interface.verify import verifyObject

from grammatista.database.createdb import createDB
from grammatista.exercise import Exercise
from grammatista.interfaces import (
    IExercise, IExerciseDefinition, IQuizDefinition)
from grammatista.testing.factory import ObjectFactory


class TestExerciseDefinition(unittest.TestCase):

    def setUp(self):
        createDB()
        self.factory = ObjectFactory()
        self.exercise_definition = self.factory.make_exercise_definition()

    def test_interface(self):
        # ExerciseDefinition implements IExerciseDefinition.
        verifyObject(IExerciseDefinition, self.exercise_definition)

    def test_inflection_forms_empty(self):
        # An exercise has no inflection forms to begin with.
        self.assertEqual(list(self.exercise_definition.inflection_forms), [])

    def test_add_inflection_form(self):
        # add_inflection_form() adds an inflection form.
        inflection_form = self.factory.make_inflection_form()
        self.exercise_definition.add_inflection_form(inflection_form)
        self.assertEqual(
            set(self.exercise_definition.inflection_forms),
            set([inflection_form]))


class TestQuizDefinition(unittest.TestCase):

    def setUp(self):
        createDB()
        self.factory = ObjectFactory()
        self.quiz_definition = self.factory.make_quiz_definition()

    def test_interface(self):
        # QuizDefinition implements IQuizDefinition.
        verifyObject(IQuizDefinition, self.quiz_definition)


class TestExercise(unittest.TestCase):

    def setUp(self):
        createDB()
        self.factory = ObjectFactory()

    def test_interface(self):
        # Exercise implements IExercise.
        exercise = self.factory.make_exercise()
        verifyObject(IExercise, exercise)

    def test_create_no_word(self):
        # If no word is provided, one of the word category's words is
        # used.
        word_category = self.factory.make_word_category()
        inflection_form_group = self.factory.make_inflection_form_group(
            language=word_category.language,
            object_string_ids=[u'inflection_a', u'inflection_b'])
        inflection_form_a, inflection_form_b = (
            word_category.add_inflection_forms(inflection_form_group))
        definition = self.factory.make_exercise_definition(
            word_category=word_category)
        definition.add_inflection_form(inflection_form_b)
        word = word_category.add_word(u'word')
        word.set_inflected_value(inflection_form_b, u'inflected-word')
        exercise = Exercise(definition)
        self.assertEqual(exercise.word, word)

    def test_create_with_word(self):
        # If a word is provided, that one is used.
        word_category = self.factory.make_word_category()
        inflection_form_group = self.factory.make_inflection_form_group(
            language=word_category.language,
            object_string_ids=[u'inflection_a', u'inflection_b'])
        inflection_form_a, inflection_form_b = (
            word_category.add_inflection_forms(inflection_form_group))
        definition = self.factory.make_exercise_definition(
            word_category=word_category)
        definition.add_inflection_form(inflection_form_b)
        # Create a bunch of words that the exercise could use.
        for index in range(100):
            word = word_category.add_word(u'word-%s' % index)
            word.set_inflected_value(inflection_form_b, unicode(index))
        another_word = word_category.add_word(u'another-word')
        another_word.set_inflected_value(
            inflection_form_b, u'another-inflected-word')
        exercise = Exercise(definition, word=another_word)
        self.assertEqual(exercise.word, another_word)

    def test_base_word(self):
        # base_word contains the basic form value of the word.
        word_category = self.factory.make_word_category()
        inflection_form_group = self.factory.make_inflection_form_group(
            language=word_category.language,
            object_string_ids=[u'inflection_a', u'inflection_b'])
        inflection_form_a, inflection_form_b = (
            word_category.add_inflection_forms(inflection_form_group))
        definition = self.factory.make_exercise_definition(
            word_category=word_category)
        definition.add_inflection_form(inflection_form_b)
        word = word_category.add_word(u'basic-form')
        word.set_inflected_value(
            inflection_form_b, u'inflected-word')
        exercise = Exercise(definition, word=word)
        self.assertEqual(exercise.base_word, u'basic-form')

    def test_create_no_inflection_form(self):
        # If no inflection form is provided, one of the definition's
        # inflection form is chosen.
        word_category = self.factory.make_word_category()
        inflection_form_group = self.factory.make_inflection_form_group(
            language=word_category.language,
            object_string_ids=[u'inflection_a', u'inflection_b'])
        inflection_form_a, inflection_form_b = (
            word_category.add_inflection_forms(inflection_form_group))
        definition = self.factory.make_exercise_definition(
            word_category=word_category)
        definition.add_inflection_form(inflection_form_b)
        word = word_category.add_word(u'basic-form')
        word.set_inflected_value(
            inflection_form_b, u'inflected-word')
        exercise = Exercise(definition)
        self.assertEqual(exercise.inflection_form, inflection_form_b)

    def test_create_with_inflection_form(self):
        # If an inflection form is provided, that one is used.
        word_category = self.factory.make_word_category()
        inflection_form_group = self.factory.make_inflection_form_group(
            language=word_category.language,
            object_string_ids=[u'inflection_a', u'inflection_b', u'specific'])
        inflection_form_a, inflection_form_b, specific_one = (
            word_category.add_inflection_forms(inflection_form_group))
        definition = self.factory.make_exercise_definition(
            word_category=word_category)
        definition.add_inflection_form(inflection_form_b)
        definition.add_inflection_form(specific_one)
        word = word_category.add_word(u'basic-form')
        word.set_inflected_value(specific_one, u'inflected-word')
        exercise = Exercise(definition, inflection_form=specific_one)
        self.assertEqual(exercise.inflection_form, specific_one)

    def test_correct_answer(self):
        # The correct answer is provided in the correctAnswer
        # attribute.
        word_category = self.factory.make_word_category()
        inflection_form_group = self.factory.make_inflection_form_group(
            language=word_category.language,
            object_string_ids=[u'inflection_a', u'inflection_b'])
        inflection_form_a, inflection_form_b = (
            word_category.add_inflection_forms(inflection_form_group))
        definition = self.factory.make_exercise_definition(
            word_category=word_category)
        definition.add_inflection_form(inflection_form_b)
        word = word_category.add_word(u'basic-form')
        word.set_inflected_value(
            inflection_form_b, u'inflected-word')
        exercise = Exercise(
            definition, word=word, inflection_form=inflection_form_b)
        self.assertEqual(exercise.correct_answer, u'inflected-word')

        # You can also check the answer using the check_answer method.
        self.assertTrue(exercise.check_answer(u'inflected-word'))
        self.assertFalse(exercise.check_answer(u'wrong-answer'))
