##############################################################################
#
# Copyright (c) 2008 Bjorn Tillenius
#
# This file is part of Grammatista.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""Tests for the Grammatista main object."""

import unittest

from zope.interface.verify import verifyObject

from grammatista import Grammatista
from grammatista.database.createdb import createDB
from grammatista.interfaces import IGrammatista
from grammatista.testing.factory import ObjectFactory


class TestGrammatista(unittest.TestCase):

    def setUp(self):
        createDB()
        self.grammatista = Grammatista()
        self.factory = ObjectFactory()

    def test_interface(self):
        # Grammatista implements IGrammatista.
        verifyObject(IGrammatista, self.grammatista)

    def test_languages_empty(self):
        # The languages attribute returns an empty list if there are no
        # languages in the DB.
        self.assertEqual(list(self.grammatista.languages), [])

    def test_languages_non_empty(self):
        # The langugaes attribute returns a list of the langauges in the
        # DB.
        lang1 = self.factory.make_language()
        lang2 = self.factory.make_language()
        self.assertEqual(set(self.grammatista.languages), set([lang1, lang2]))

    def test_get_language_existing(self):
        # get_language() returns the language with the given string_id.
        lang = self.factory.make_language(string_id=u'test-lang')
        self.assertEqual(self.grammatista.get_language(u'test-lang'), lang)

    def test_get_language_existing(self):
        # get_language() returns None if the language doesn't exist.
        self.assertEqual(self.grammatista.get_language(u'test-lang'), None)

    def test_add_language(self):
        # add_language() adds a new language to the DB and returns it.
        lang = self.grammatista.add_language(u'test-lang', u'Test Lang')
        self.assertEquals(self.grammatista.get_language(u'test-lang'), lang)
        self.assertEquals(lang.string_id, u'test-lang')
        self.assertEquals(lang.name, u'Test Lang')

    def test_remove_language(self):
        # remove_language removes a language from the DB.
        lang = self.factory.make_language(string_id=u'test-lang')
        self.assertTrue(
            self.grammatista.get_language(u'test-lang') is not None)
        self.grammatista.remove_language(u'test-lang')
        self.assertEquals(self.grammatista.get_language(u'test-lang'), None)

    def test_remove_language_nonexisting(self):
        # Removing a nonexisting language is a noop.
        self.grammatista.remove_language(u'test-lang')
        self.assertEquals(self.grammatista.get_language(u'test-lang'), None)
