##############################################################################
#
# Copyright (c) 2008 Bjorn Tillenius
#
# This file is part of Grammatista.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""Tests for InflectionForm and related classes."""

import unittest

from zope.interface.verify import verifyObject

from grammatista.database.createdb import createDB
from grammatista.inflection import (
    AmbiguousInflectionFormException, generate_inflection_forms)
from grammatista.interfaces import (
    IInflection, IInflectionForm, IInflectionFormGroup, IInflectionFormObject)
from grammatista.testing.factory import ObjectFactory


class TestInflectionFormGroup(unittest.TestCase):

    def setUp(self):
        createDB()
        self.factory = ObjectFactory()
        self.form_group = self.factory.make_inflection_form_group()

    def test_interface(self):
        # InflectionFormGroup implements IInflectionFormGroup.
        verifyObject(IInflectionFormGroup, self.form_group)

    def test_add_object(self):
        # add_object() adds a new InflectionFormObject to the group.
        form_object = self.form_group.add_object(u'object', u'Object')
        self.assertEqual(form_object.string_id, u'object')
        self.assertEqual(form_object.name, u'Object')
        self.assertEqual(form_object.group, self.form_group)
        self.assertEqual(self.form_group.get_object(u'object'), form_object)

    def test_get_object(self):
        # get_object() returns a form object by its string_id.
        form_object = self.factory.make_inflection_form_object(
            group=self.form_group)

    def test_objects_empty(self):
        # If there are no objects, objects is empty.
        self.assertEqual(list(self.form_group.objects), [])

    def test_objects(self):
        # objects contains all the objects of the group. It's ordered by
        # the order the objects were added.
        first_object = self.factory.make_inflection_form_object(
            group=self.form_group)
        second_object = self.factory.make_inflection_form_object(
            group=self.form_group)
        object_in_other_group = self.factory.make_inflection_form_object()
        self.assertEqual(
            set(self.form_group.objects), set([first_object, second_object]))


class TestInflectionForm(unittest.TestCase):

    def setUp(self):
        createDB()
        self.factory = ObjectFactory()
        self.inflection_form = self.factory.make_inflection_form()

    def test_interface(self):
        # InflectionForm implements IInflectionForm.
        verifyObject(IInflectionForm, self.inflection_form)

    def test_objects_empty(self):
        # objects is empty if there are no objects in the inflection form.
        self.assertEqual(list(self.inflection_form.objects), [])

    def test_objects(self):
        # objects contains the object of the inflection from.
        first_object = self.factory.make_inflection_form_object()
        second_object = self.factory.make_inflection_form_object()
        self.inflection_form.add_object(first_object)
        self.inflection_form.add_object(second_object)
        self.assertEqual(
            list(self.inflection_form.objects), [first_object, second_object])

    def test_add_object_ambiguous(self):
        # An inflection form consists of objects from different
        # inflection form groups (e.g., case, gender). If two objects
        # from the same group is added, an
        # AmbiguousInflectionFormException is raised.
        inflection_object = self.factory.make_inflection_form_object()
        self.inflection_form.add_object(inflection_object)
        another_inflection_object  = self.factory.make_inflection_form_object(
            group=inflection_object.group)
        self.assertRaises(
            AmbiguousInflectionFormException,
            self.inflection_form.add_object, another_inflection_object)

    def test_string_id(self):
        # The string_id of the inflection form gets generated by joining
        # the objects' string_ids with a comma.
        self.inflection_form.add_object(
            self.factory.make_inflection_form_object())
        self.inflection_form.add_object(
            self.factory.make_inflection_form_object())
        self.assertEquals(
            self.inflection_form.string_id,
            ', '.join(ob.string_id for ob in self.inflection_form.objects))

    def test_name(self):
        # The name of the inflection form gets generated by
        # joining the objects' names with a comma.
        self.inflection_form.add_object(
            self.factory.make_inflection_form_object())
        self.inflection_form.add_object(
            self.factory.make_inflection_form_object())
        self.assertEquals(
            self.inflection_form.name,
            ', '.join(ob.name for ob in self.inflection_form.objects))


class TestGenerateInflectionForms(unittest.TestCase):

    def setUp(self):
        createDB()
        self.factory = ObjectFactory()

    def test_single(self):
        # Passing in one group generates one inflection form for each
        # object in the group.
        form_group = self.factory.make_inflection_form_group()
        form_group.add_object(u'foo', u'foo')
        form_group.add_object(u'bar', u'foo')
        inflection_forms = generate_inflection_forms([form_group])
        self.assertEquals(
            set(form.string_id for form in inflection_forms),
            set([u'foo', u'bar']))

    def test_multiple(self):
        # Passing in more than one group, creates one inflection form
        # for each possible permutation.
        form_group = self.factory.make_inflection_form_group()
        form_group.add_object(u'foo', u'foo')
        form_group.add_object(u'bar', u'foo')
        another_form_group = self.factory.make_inflection_form_group()
        another_form_group.add_object(u'baz', u'baz')
        inflection_forms = generate_inflection_forms(
            [form_group, another_form_group])
        self.assertEquals(
            set(form.string_id for form in inflection_forms),
            set([u'foo, baz', u'bar, baz']))


class TestInflectionFormObject(unittest.TestCase):

    def setUp(self):
        createDB()
        self.factory = ObjectFactory()
        self.form_object = self.factory.make_inflection_form_object()

    def test_interface(self):
        # InflectionForm implements IInflectionForm.
        verifyObject(IInflectionFormObject, self.form_object)


class TestInflection(unittest.TestCase):

    def setUp(self):
        createDB()
        self.factory = ObjectFactory()
        self.inflection = self.factory.make_inflection()

    def test_interface(self):
        # Inflection implements IInflection.
        verifyObject(IInflection, self.inflection)

    def test_input_regexps(self):
        # Initially, there are no input regexps.
        self.assertEqual(list(self.inflection.input_regexps), [])

    def test_add_input_regexp(self):
        # add_input_regexp() adds a new input RegExp.
        self.inflection.add_input_regexp(u'foo')
        self.inflection.add_input_regexp(u'bar')
        self.assertEqual(
            set(regexp.value for regexp in self.inflection.input_regexps),
            set([u'foo', u'bar']))

    def test_add_output_form(self):
        # Initially, there are no output forms.
        self.assertEqual(list(self.inflection.output_inflection_forms), [])

    def test_add_output_form(self):
        # add_output_form adds things to the output_inflection_forms
        # list.
        output_form = self.factory.make_inflection_form()
        another_output_form = self.factory.make_inflection_form(
            language=output_form.language)
        self.inflection.add_output_form(output_form)
        self.inflection.add_output_form(another_output_form)
        self.assertEqual(
            set(self.inflection.output_inflection_forms),
            set([output_form, another_output_form]))

    def test_morph_properties(self):
        # Initially, there are no morphological properties.
        self.assertEqual(list(self.inflection.morph_properties), [])

    def test_add_morph_properties(self):
        # add_morph_property adds things to the morph_properties list.
        form_object = self.factory.make_inflection_form_object()
        another_form_object = self.factory.make_inflection_form_object(
            group=form_object.group)
        self.inflection.add_morph_property(form_object)
        self.inflection.add_morph_property(another_form_object)
        self.assertEqual(
            set(self.inflection.morph_properties),
            set([form_object, another_form_object]))

    def test_inflect_match(self):
        # If one of the input regexps match the given word, the
        # inflected string is returned.
        class FakeWord:
            def get_inflected_value(self, inflection_form):
                return u'foo'
        word = FakeWord()
        self.inflection.output_regexp = ur'\1 bar'
        self.inflection.add_input_regexp(u'^(foo)$')
        self.assertEqual(self.inflection.inflect(word), u'foo bar')

    def test_inflect_no_match(self):
        # If none of the input regexps match the given word, None is
        # returned.
        class FakeWord:
            def get_inflected_value(self, inflection_form):
                return u'nomatch'
        word = FakeWord()
        self.inflection.output_regexp = ur'\1 bar'
        self.inflection.add_input_regexp(u'^(foo)$')
        self.assertEqual(self.inflection.inflect(word), None)


class TestRegExp(unittest.TestCase):

    def setUp(self):
        createDB()
        self.factory = ObjectFactory()

    def test_compiled_regexp(self):
        # A compiled version of the reg exp is available in
        # compiled_regexp.
        regexp = self.factory.make_regexp(value=u'foo')
        self.assertTrue(regexp.compiled_regexp.match(u'foo') is not None)

    def test_match_match(self):
        # match() tells you whether the string matches the regexp.
        regexp = self.factory.make_regexp(value=u'^foo.*')
        self.assertTrue(regexp.match(u'foobar'))

    def test_match_match_no_match(self):
        # match() tells you whether the string matches the regexp.
        regexp = self.factory.make_regexp(value=u'^bar.*')
        self.assertFalse(regexp.match(u'foobar'))

    def test_sub(self):
        # sub() applies the regexp on the string and do any
        # substitutions in the inflection's output regexp.
        regexp = self.factory.make_regexp(value=u'^foo(.*)')
        regexp.inflection.output_regexp = ur'\1baz'
        self.assertEqual(regexp.sub(u'foobar'), u'barbaz')
