##############################################################################
#
# Copyright (c) 2008 Bjorn Tillenius
#
# This file is part of Grammatista.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""Tests for the Language."""

import unittest

from zope.interface.verify import verifyObject

from grammatista.database.createdb import createDB
from grammatista.interfaces import ILanguage
from grammatista.testing.factory import ObjectFactory


class TestLanguage(unittest.TestCase):

    def setUp(self):
        createDB()
        self.factory = ObjectFactory()
        self.language = self.factory.make_language()

    def test_interface(self):
        # Language implements ILanguage.
        verifyObject(ILanguage, self.language)

    def test_add_word_category(self):
        # add_word_category() creates a word category with the given string_id.
        word_category = self.language.add_word_category(
            u'word-category', u'Word Category')
        self.assertEqual(word_category.string_id, u'word-category')
        self.assertEqual(word_category.name, u'Word Category')
        self.assertEqual(
            self.language.get_word_category(u'word-category'), word_category)

    def test_get_word_category(self):
        # get_word_category() returns a word category by string_id.
        word_category = self.factory.make_word_category(
            language=self.language)
        self.assertEqual(
            self.language.get_word_category(word_category.string_id),
            word_category)

    def test_get_word_category_nonexisting(self):
        # get_word_category() returns None if there is no such word
        # category for a language.
        word_category_in_other_language = self.factory.make_word_category()
        self.assertEqual(
            self.language.get_word_category(
                word_category_in_other_language.string_id),
            None)

    def test_word_categories_empty(self):
        # If there are no word categories for the language,
        # word_categories is # empty.
        self.assertEqual(list(self.language.word_categories), [])

    def test_word_categories(self):
        # word_categories contain all the word categories in the
        # language.
        word_category1 = self.factory.make_word_category(
            language=self.language)
        word_category2 = self.factory.make_word_category(
            language=self.language)
        self.assertEqual(
            set(self.language.word_categories),
            set([word_category1, word_category2]))

    def test_get_inflection_form_group(self):
        # get_inflection_form_group() returns an inflection for group by
        # its string_id.
        form_group = self.factory.make_inflection_form_group(
            language=self.language)
        self.assertEqual(
            self.language.get_inflection_form_group(form_group.string_id),
            form_group)

    def test_get_inflection_form_group_nonexisting(self):
        # get_inflection_form_group() returns None if there is no such
        # inflection from group.
        form_group_in_other_language = (
            self.factory.make_inflection_form_group())
        self.assertEqual(
            self.language.get_inflection_form_group(
                form_group_in_other_language.string_id),
            None)

    def test_add_inflection_from_group(self):
        # add_inflection_form_group() creates an inflection form group
        # with the given string_id.
        form_group = self.language.add_inflection_form_group(
            u'form-group', u'Form Group')
        self.assertEqual(form_group.string_id, u'form-group')
        self.assertEqual(form_group.name, u'Form Group')
        self.assertEqual(
            self.language.get_inflection_form_group(u'form-group'),
            form_group)
        # It has a reference back to the language.
        self.assertEqual(form_group.language, self.language)

    def test_inflection_form_groups_empty(self):
        # If there are no inflection form groups, inflection_form_groups
        # is empty.
        self.assertEquals(list(self.language.inflection_form_groups), [])

    def test_word_lists_empty(self):
        # If there are no word lists for the language, word_lists is
        # empty.
        self.assertEqual(list(self.language.word_lists), [])

    def test_word_lists(self):
        # word_lists contains all the word categories in the language.
        word_list1 = self.factory.make_word_list(
            language=self.language)
        word_list2 = self.factory.make_word_list(
            language=self.language)
        self.assertEqual(
            set(self.language.word_lists), set([word_list1, word_list2]))

    def test_add_word_list(self):
        # add_word_list() adds a new word list to the language.
        word_list = self.language.add_word_list(u'word-list', u'Word List')
        self.assertEqual(word_list.string_id, u'word-list')
        self.assertEqual(word_list.name, u'Word List')
        self.assertEqual(set(self.language.word_lists), set([word_list]))

    def test_exercise_definitions(self):
        # exercise_definitions contains all the definitions in the
        # language, across the language's word categories.
        one_word_category = self.factory.make_word_category(
            language=self.language)
        one_exercise_definition = self.factory.make_exercise_definition(
            word_category=one_word_category)
        another_word_category = self.factory.make_word_category(
            language=self.language)
        another_exercise_definition = self.factory.make_exercise_definition(
            word_category=another_word_category)
        definition_in_other_language = self.factory.make_exercise_definition()
        self.assertEquals(
            set(self.language.exercise_definitions),
            set([one_exercise_definition, another_exercise_definition]))

    def test_quiz_definitions(self):
        # quiz_definitions contains all the definitions in the
        # language, across the language's word categories.
        one_word_category = self.factory.make_word_category(
            language=self.language)
        one_exercise_definition = self.factory.make_exercise_definition(
            word_category=one_word_category)
        one_quiz_definition = self.factory.make_quiz_definition(
            exercise_definition=one_exercise_definition)
        another_word_category = self.factory.make_word_category(
            language=self.language)
        another_exercise_definition = self.factory.make_exercise_definition(
            word_category=another_word_category)
        another_quiz_definition = self.factory.make_quiz_definition(
            exercise_definition=another_exercise_definition)
        definition_in_other_language = self.factory.make_quiz_definition()
        self.assertEquals(
            set(self.language.quiz_definitions),
            set([one_quiz_definition, another_quiz_definition]))
