##############################################################################
#
# Copyright (c) 2008 Bjorn Tillenius
#
# This file is part of Grammatista.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""Tests for Word and related things."""

import unittest

from zope.interface.verify import verifyObject

from grammatista.database.createdb import createDB
from grammatista.interfaces import IWord
from grammatista.testing.factory import ObjectFactory


class TestWord(unittest.TestCase):

    def setUp(self):
        createDB()
        self.factory = ObjectFactory()
        self.word = self.factory.make_word()

    def test_interface(self):
        # Word implements IWord.
        verifyObject(IWord, self.word)

    def test_set_inflected_value_existing(self):
        # set_inflected_value() sets a word for a specific inflection
        # form. If there is already an existing value for that form,
        # that InflectedWord instance is modified.
        inflected_word = self.word.set_inflected_value(
            self.word.word_category.basic_form, u'modified-inflected-value')
        self.assertEqual(inflected_word.word, self.word)
        self.assertEqual(
            inflected_word.inflection_form,
            self.word.word_category.basic_form)
        self.assertEqual(inflected_word.value, u'modified-inflected-value')
        self.assertEqual(
            self.word.get_inflected_value(self.word.word_category.basic_form),
            u'modified-inflected-value')
        inflected_values = [
            inflected_word.value
            for inflected_word in self.word.inflected_words
            ]
        self.assertEqual(
            inflected_values, [u'modified-inflected-value'])

    def test_set_inflected_value_new(self):
        # set_inflected_value() sets a word for a specific inflection
        # form. If there is already an existing value for that form,
        # that InflectedWord instance is modified.
        new_inflection_form = self.factory.make_inflection_form(
            language=self.word.word_category.language)
        self.word.word_category.add_inflection_form(new_inflection_form)
        inflected_word = self.word.set_inflected_value(
            new_inflection_form, u'new-inflected-value')
        self.assertEqual(inflected_word.word, self.word)
        self.assertEqual(
            inflected_word.inflection_form, new_inflection_form)
        self.assertEqual(inflected_word.value, u'new-inflected-value')
        self.assertEqual(
            self.word.get_inflected_value(new_inflection_form),
            u'new-inflected-value')
        inflected_values = [
            inflected_word.value
            for inflected_word in self.word.inflected_words
            ]
        self.assertEqual(
            inflected_values, [self.word.name, u'new-inflected-value'])

        # Using None as the value deletes the InflectedWord.
        self.assertEqual(
            self.word.set_inflected_value(new_inflection_form, None),
            None)
        self.assertEqual(
            self.word.get_inflected_value(new_inflection_form), None)
        inflected_values = [
            inflected_word.value
            for inflected_word in self.word.inflected_words
            ]
        self.assertEqual(inflected_values, [self.word.name])

    def test_get_inflected_value_inflect(self):
        # get_inflected_value() gets a word for a specific inflection
        # form. If there isn't already an existing InflectedWord for
        # that form, the value is inflected using the inflection rules.
        new_inflection_form = self.factory.make_inflection_form(
            language=self.word.word_category.language)
        self.word.word_category.add_inflection_form(new_inflection_form)
        basic_word_value = self.word.get_inflected_value(
            self.word.word_category.basic_form)
        self.word.word_category.add_inflection(
            self.word.morph_properties, self.word.word_category.basic_form,
            [basic_word_value],
            [new_inflection_form], ur'inflected')
        self.assertEqual(
            self.word.get_inflected_value(new_inflection_form), u'inflected')

    def test_get_inflected_value_inflect_none(self):
        # If the value can't be inflected, None is returned.
        new_inflection_form = self.factory.make_inflection_form(
            language=self.word.word_category.language)
        self.word.word_category.add_inflection_form(new_inflection_form)
        self.word.word_category.add_inflection(
            self.word.morph_properties, self.word.word_category.basic_form,
            [u'no-match'],
            [new_inflection_form], ur'inflected')
        self.assertEqual(
            self.word.get_inflected_value(new_inflection_form), None)

