##############################################################################
#
# Copyright (c) 2008 Bjorn Tillenius
#
# This file is part of Grammatista.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""Tests for WordCategory and related classes."""

import unittest

from zope.interface.verify import verifyObject

from grammatista.database.createdb import createDB
from grammatista.interfaces import IWordCategory
from grammatista.testing.factory import ObjectFactory


class TestWordCategory(unittest.TestCase):

    def setUp(self):
        createDB()
        self.factory = ObjectFactory()
        self.word_category = self.factory.make_word_category()

    def test_interface(self):
        # WordCategory implements IWordCategory.
        verifyObject(IWordCategory, self.word_category)

    def test_basic_form_none(self):
        # A word category doesn't have a basic form to begin with.
        self.assertEqual(self.word_category.basic_form, None)

    def test_basic_form_set_to_first_inflection_form(self):
        # The basic form is set to the first inflection form added to
        # the word category.
        first_inflection_form = self.factory.make_inflection_form(
            language=self.word_category.language)
        second_inflection_form = self.factory.make_inflection_form(
            language=self.word_category.language)
        self.word_category.add_inflection_form(first_inflection_form)
        self.word_category.add_inflection_form(second_inflection_form)
        self.assertEqual(self.word_category.basic_form, first_inflection_form)

    def test_add_inflection_form(self):
        # add_inflection_form_group() adds things to the
        # inflection_forms attribute.
        first_inflection_form = self.factory.make_inflection_form(
            language=self.word_category.language)
        second_inflection_form = self.factory.make_inflection_form(
            language=self.word_category.language)
        self.word_category.add_inflection_form(first_inflection_form)
        self.word_category.add_inflection_form(second_inflection_form)
        self.assertEqual(
            set(self.word_category.inflection_forms),
            set([first_inflection_form, second_inflection_form]))

    def test_add_morph_property_group(self):
        # add_morp_property_group() adds things to the
        # morph_property_groups attribute.
        first_inflection_form = self.factory.make_inflection_form_group(
            language=self.word_category.language)
        second_inflection_form = self.factory.make_inflection_form_group(
            language=self.word_category.language)
        self.word_category.add_morph_property_group(first_inflection_form)
        self.word_category.add_morph_property_group(second_inflection_form)
        self.assertEqual(
            list(self.word_category.morph_property_groups),
            [first_inflection_form, second_inflection_form])

    def test_morph_properties_empty(self):
        # morph_properties is empty to begin with.
        self.assertEqual(list(self.word_category.morph_properties), [])

    def test_morph_properties(self):
        # morph_properties contains all the objects of the morphological
        # groups of the word category.
        first_inflection_form = self.factory.make_inflection_form_group(
            language=self.word_category.language,
            object_string_ids=[u'foo', u'bar'])
        second_inflection_form = self.factory.make_inflection_form_group(
            language=self.word_category.language,
            object_string_ids=[u'baz'])
        self.word_category.add_morph_property_group(first_inflection_form)
        self.word_category.add_morph_property_group(second_inflection_form)
        self.assertEqual(
            set(ob.string_id for ob in self.word_category.morph_properties),
            set([u'foo', u'bar', u'baz']))

    def test_add_inflection_forms(self):
        # add_inflection_forms() adds the permutation of all the objects
        # in the groups.
        first_form_group = self.factory.make_inflection_form_group(
            language=self.word_category.language,
            object_string_ids=[u'foo', u'bar'])
        second_form_group = self.factory.make_inflection_form_group(
            language=self.word_category.language,
            object_string_ids=[u'baz'])
        self.word_category.add_inflection_forms(
            first_form_group, second_form_group)
        added_form_string_ids = [
            inflection_form.string_id
            for inflection_form in self.word_category.inflection_forms
            ]
        self.assertEqual(
            set(added_form_string_ids), set([u'foo, baz', u'bar, baz']))

    def test_get_inflection_forms(self):
        # get_inflection_forms() returns the inflection forms that has
        # objects with all the given string_ids.
        first_form_group = self.factory.make_inflection_form_group(
            language=self.word_category.language,
            object_string_ids=[u'foo', u'bar'])
        second_form_group = self.factory.make_inflection_form_group(
            language=self.word_category.language,
            object_string_ids=[u'baz'])
        self.word_category.add_inflection_forms(
            first_form_group, second_form_group)
        self.assertEquals(
            [ob.string_id
             for ob in self.word_category.get_inflection_forms(u'foo')],
            [u'foo, baz'])
        self.assertEquals(
            set(
                ob.string_id
                for ob in self.word_category.get_inflection_forms(u'baz')),
            set([u'foo, baz', u'bar, baz']))
        self.assertEquals(
            set(
                ob.string_id
                for ob
                in self.word_category.get_inflection_forms(u'foo', u'baz')),
            set([u'foo, baz']))

    def test_get_inflection_form(self):
        # get_inflection_form() returns the inflection form that has
        # objects with all the given string_ids.
        first_form_group = self.factory.make_inflection_form_group(
            language=self.word_category.language,
            object_string_ids=[u'foo', u'bar'])
        second_form_group = self.factory.make_inflection_form_group(
            language=self.word_category.language,
            object_string_ids=[u'baz'])
        self.word_category.add_inflection_forms(
            first_form_group, second_form_group)
        self.assertEquals(
             self.word_category.get_inflection_form(u'foo', u'baz').string_id,
            u'foo, baz')

    def test_add_inflection(self):
        # add_inflection() adds a new inflection.
        case = self.factory.make_inflection_form_group(
            language=self.word_category.language,
            object_string_ids=[u'Nom', u'Gen'])
        gender = self.factory.make_inflection_form_group(
            language=self.word_category.language,
            object_string_ids=[u'Masc', u'Fem'])
        self.word_category.add_inflection_forms(case)
        nom, gen = self.word_category.inflection_forms
        inflection = self.word_category.add_inflection(
            [gender.get_object(u'Masc')], nom, [u'foo(.*)'], [gen], ur'\1bar')
        self.assertEqual(inflection.word_category, self.word_category)
        self.assertEqual(inflection.input_inflection_form, nom)
        self.assertEqual(
            list(inflection.morph_properties), [gender.get_object(u'Masc')])
        self.assertEqual(
            [regexp.value for regexp in inflection.input_regexps],
            [u'foo(.*)'])
        self.assertEqual(list(inflection.output_inflection_forms), [gen])
        self.assertEqual(inflection.output_regexp, ur'\1bar')

    def test_add_inflections(self):
        # add_inflections() adds a new inflections, using the same input
        # for a sequence of output forms and regexps.
        case = self.factory.make_inflection_form_group(
            language=self.word_category.language,
            object_string_ids=[u'Nom', u'Gen', u'Dat'])
        gender = self.factory.make_inflection_form_group(
            language=self.word_category.language,
            object_string_ids=[u'Masc', u'Fem'])
        self.word_category.add_inflection_forms(case)
        nom, gen, dat = self.word_category.inflection_forms
        inflections = self.word_category.add_inflections(
            [gender.get_object(u'Masc')], nom, [u'foo(.*)'],
            [([gen], ur'\1gen'), ([dat], ur'\1dat')])
        for inflection in inflections:
            self.assertEqual(inflection.word_category, self.word_category)
            self.assertEqual(inflection.input_inflection_form, nom)
            self.assertEqual(
                list(inflection.morph_properties),
                [gender.get_object(u'Masc')])
            self.assertEqual(
                [regexp.value for regexp in inflection.input_regexps],
                [u'foo(.*)'])
        self.assertEqual(list(inflections[0].output_inflection_forms), [gen])
        self.assertEqual(inflections[0].output_regexp, ur'\1gen')
        self.assertEqual(list(inflections[1].output_inflection_forms), [dat])
        self.assertEqual(inflections[1].output_regexp, ur'\1dat')

    def test_add_word(self):
        # add_word() adds a word with the given morphological
        # properties, and sets the basic form to be the passed in value.
        case = self.factory.make_inflection_form_group(
            language=self.word_category.language,
            object_string_ids=[u'Nom', u'Gen', u'Dat'])
        gender = self.factory.make_inflection_form_group(
            language=self.word_category.language,
            object_string_ids=[u'Masc', u'Fem'])
        word = self.word_category.add_word(
            u'foobar', gender.get_object(u'Masc'))
        self.assertEqual(word.word_category, self.word_category)
        self.assertEqual(
            word.get_inflected_value(self.word_category.basic_form),
            u'foobar')
        self.assertEqual(
            list(word.morph_properties), [gender.get_object(u'Masc')])

    def test_words(self):
        # added words are available in the words attribute.
        first_word = self.factory.make_word(word_category=self.word_category)
        second_word = self.factory.make_word(word_category=self.word_category)
        self.assertEqual(
            list(self.word_category.words), [first_word, second_word])

    def test_get_word(self):
        # get_word() returns the Word which has an inflection as the
        # given string.
        word = self.factory.make_word(
            value=u'foobar', word_category=self.word_category)
        self.assertEqual(self.word_category.get_word(u'foobar'), word)

    def test_get_word_none(self):
        # get_word() returns None if no word can be found.
        word = self.factory.make_word(
            value=u'foobar', word_category=self.word_category)
        self.assertEqual(self.word_category.get_word(u'not-matching'), None)

    def test_add_exercise_definition(self):
        # add_exercise_definition() adds a new exercise definition.
        inflection_form = self.factory.make_inflection_form()
        definition = self.word_category.add_exercise_definition(
            u'foo', u'Foo', inflection_form)
        self.assertEqual(definition.word_category, self.word_category)
        self.assertEqual(definition.string_id, u'foo')
        self.assertEqual(definition.name, u'Foo')
        self.assertEqual(
            set(definition.inflection_forms),
            set([inflection_form]))
        self.assertEqual(
            set(self.word_category.exercise_definitions),
            set([definition]))

    def test_get_exercise_definition(self):
        # get_exercise_definition() returns an exercise definition by
        # string id.
        definition = self.factory.make_exercise_definition(
            u'foo', word_category=self.word_category)
        self.assertEqual(
            self.word_category.get_exercise_definition(u'foo'), definition)

    def test_get_exercise_definition_none(self):
        # If no such definition with the given id exists, None is
        # returned.
        self.assertEqual(
            self.word_category.get_exercise_definition(u'no-match'), None)

    def test_add_quiz_definition(self):
        # add_quz_definition() adds a new quiz definition.
        inflection_form = self.factory.make_inflection_form()
        exercise_definition = self.factory.make_exercise_definition(
            u'foo', word_category=self.word_category)
        quiz_definition = self.word_category.add_quiz_definition(
            u'foo', u'Foo', exercise_definition, 42)
        self.assertEqual(
            quiz_definition.exercise_definition, exercise_definition)
        self.assertEqual(quiz_definition.string_id, u'foo')
        self.assertEqual(quiz_definition.name, u'Foo')
        self.assertEqual(quiz_definition.number_of_exercises, 42)
        self.assertEqual(
            set(self.word_category.quiz_definitions), set([quiz_definition]))

    def test_get_quiz_definition(self):
        # get_quiz_definition() returns a quiz definition by string id.
        exercise_definition = self.factory.make_exercise_definition(
            u'foo', word_category=self.word_category)
        quiz_definition = self.factory.make_quiz_definition(
            u'bar', exercise_definition=exercise_definition)
        self.assertEqual(
            self.word_category.get_quiz_definition(u'bar'), quiz_definition)

    def test_get_quiz_definition_none(self):
        # If no such definition with the given id exists, None is
        # returned.
        self.assertEqual(
            self.word_category.get_quiz_definition(u'no-match'), None)
