##############################################################################
#
# Copyright (c) 2004,2008 Bjorn Tillenius
#
# This file is part of Grammatista.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""Word related classes and functions."""
import zope.interface

from storm.base import Storm
from storm.properties import Int, Unicode
from storm.references import Reference, ReferenceSet
from storm.store import Store

from grammatista.exercise import ExerciseDefinition, QuizDefinition
from grammatista.inflection import (
    generate_inflection_forms,
    Inflection,
    InflectedWord,
    )
from grammatista.interfaces import IWordCategory, IWord, IWordList
from grammatista.references import InflectionOutputInflectionForm


class WordCategory(Storm):
    """A word category."""

    zope.interface.implements(IWordCategory)

    __storm_table__ = "WordCategory"

    id = Int(primary=True)
    string_id = Unicode()
    name = Unicode()
    language_id = Int(name='language')
    language = Reference(language_id, 'Language.id')

    basic_form_id = Int(name='basic_form')
    basic_form = Reference(basic_form_id, 'InflectionForm.id')
    inflection_forms = ReferenceSet(
        id,
        'WordCategoryInflectionForm.word_category_id',
        'WordCategoryInflectionForm.inflection_form_id',
        'InflectionForm.id')
    morph_property_groups = ReferenceSet(
        id,
        'WordCategoryMorphPropertyGroup.word_category_id',
        'WordCategoryMorphPropertyGroup.inflection_form_group_id',
        'InflectionFormGroup.id')

    exercise_definitions = ReferenceSet(
        id, 'ExerciseDefinition.word_category_id')
    quiz_definitions = ReferenceSet(
        id, 'QuizDefinition.word_category_id')
    words = ReferenceSet(id, 'Word.word_category_id')

    def __init__(self, string_id, name, language):
        self.string_id = string_id
        self.name = name
        self.language = language

    def add_morph_property_group(self, inflection_form_group):
        self.morph_property_groups.add(inflection_form_group)
        # The ordering of the inserted inflection forms are important.
        Store.of(self).flush()

    @property
    def morph_properties(self):
        """See IWordCategory."""
        objects = []
        for group in self.morph_property_groups:
            objects.extend(group.objects)
        return objects

    def add_inflection_form(self, inflection_form):
        """Adds a single inflection form to the word category."""
        if not self.basic_form:
            self.basic_form = inflection_form
        self.inflection_forms.add(inflection_form)
        # The ordering of the inserted inflection forms are important.
        Store.of(self).flush()
        return inflection_form

    def add_inflection_forms(self, *inflection_form_groups):
        """See IWordCategory."""
        inflection_forms = list(
            generate_inflection_forms(inflection_form_groups))
        for inflection_form in inflection_forms:
            self.add_inflection_form(inflection_form)
        return inflection_forms

    def get_inflection_forms(self, *string_ids):
        """See IWordCategory."""
        L = []
        for inflection_form in self.inflection_forms:
            form_string_ids = [
                object.string_id for object in inflection_form.objects]
            match = True
            for string_id in string_ids:
                if string_id not in form_string_ids:
                    match = False
            if match:
                L.append(inflection_form)
        return L

    def get_inflection_form(self, *string_ids):
        """See IWordCategory."""
        forms = list(self.get_inflection_forms(*string_ids))
        if forms:
            return forms[0]
        else:
            return None

    def add_inflection(self, morph_properties, input_form, input_regexps,
                       output_forms, output_regexp):
        """Add an inflection and returns it."""
        inflection = Inflection(input_inflection_form=input_form,
                                output_regexp=output_regexp,
                                word_category=self)
        store = Store.of(self)
        inflection = store.add(inflection)
        for morph_property in morph_properties:
            inflection.add_morph_property(morph_property)
        for input_regexp in input_regexps:
            inflection.add_input_regexp(input_regexp)
        for output_form in output_forms:
            inflection.add_output_form(output_form)
        return inflection

    def add_inflections(self, morph_properties,
                       input_form, input_regexps,
                       output_mappings):
        """Adds several inflections at once and returns them."""
        inflections = []
        for output_forms, output_regexp in output_mappings:
            inflections.append(self.add_inflection(
                morph_properties, input_form, input_regexps, output_forms,
                output_regexp))
        return inflections

    def add_exercise_definition(self, string_id, name, *inflection_forms):
        """See IWordCategory."""
        definition = ExerciseDefinition(
            string_id=string_id, name=name, word_category=self)
        definition = Store.of(self).add(definition)
        for inflection_form in inflection_forms:
            definition.add_inflection_form(inflection_form)
        return definition

    def get_exercise_definition(self, string_id):
        """See IWordCategory."""

        for definition in self.exercise_definitions:
            if definition.string_id == string_id:
                return definition
        return None

    def add_quiz_definition(self, string_id, name, exercise_definition,
                            number_of_exercises):
        """See `IWriteWordCategory`."""
        definition = QuizDefinition(
            string_id=string_id, name=name, word_category=self,
            exercise_definition=exercise_definition,
            number_of_exercises=number_of_exercises)
        return Store.of(self).add(definition)

    def get_quiz_definition(self, string_id):
        """See IWordCategory."""
        for definition in self.quiz_definitions:
            if definition.string_id == string_id:
                return definition
        return None

    def add_word(self, value, *morph_properties):
        """Adds a new word with the specified properties and returns it."""
        word = Word(word_category=self)
        word = Store.of(self).add(word)
        for morph_property in morph_properties:
            word.add_morph_property(morph_property)
        word.set_inflected_value(self.basic_form, value)
        return word


    def get_word(self, value):
        """See IWordCategory."""
        store = Store.of(self)
        inflected_words = store.find(
            InflectedWord,
            InflectedWord.value == value,
            InflectedWord.inflection_form == self.basic_form)
        inflected_word = inflected_words.one()
        if inflected_word is not None:
            return inflected_word.word
        else:
            return None



class Word(Storm):
    """See IWord"""

    zope.interface.implements(IWord)

    __storm_table__ = "Word"

    @property
    def name(self):
        return self.get_inflected_value(self.word_category.basic_form)
    string_id = name

    id = Int(primary=True)
    word_category_id = Int(name='word_category')
    word_category = Reference(word_category_id, 'WordCategory.id')
    morph_properties = ReferenceSet(
        id,
        'WordMorphProperty.word_id',
        'WordMorphProperty.inflection_form_object_id',
        'InflectionFormObject.id')
    inflected_words = ReferenceSet(id, 'InflectedWord.word_id')

    def __init__(self, word_category):
        self.word_category = word_category

    def add_morph_property(self, inflection_form_object):
        self.morph_properties.add(inflection_form_object)

    def _inflect(self, inflection_form):
        """Tries to inflect the word to the given inflection form.

        It returns the value of the inflected word.
        If no inflection can be found, None is returned.
        """
        store = Store.of(self)
        inflections = store.find(
            Inflection,
            Inflection.word_category == self.word_category,
            Inflection.id == InflectionOutputInflectionForm.inflection_id,
            InflectionOutputInflectionForm.inflection_form_id ==
                inflection_form.id)
        for inflection in inflections:
            match = True
            for morph_property in self.morph_properties:
                if morph_property not in inflection.morph_properties:
                    match = False
            if match:
                value = inflection.inflect(self) 
                if value:
                    return  value

        return None

    def get_inflected_value(self, inflection_form):
        """Returns the value of an inflected word.

        It looks up the first inflected word which has the specified
        inflection form, and return its value.

        If it can't find a suitable word in the database, it tries to
        find an inflection and uses this to return the right value.

        If it can't find neither a suitable word nor an suitable
        inflection it returns None.
        """
        for inflected_word in self.inflected_words:
            if inflection_form == inflected_word.inflection_form:
                return inflected_word.value

        return self._inflect(inflection_form)

    def set_inflected_value(self, inflection_form, value):
        """Sets the inflected word of the specified inflection form."""
        store = Store.of(self)
        inflected_words = store.find(
            InflectedWord,
            InflectedWord.word == self,
            InflectedWord.inflection_form == inflection_form)
        if value is None:
            inflected_words.remove()
            return
        inflected_word = inflected_words.one()
        if inflected_word is None:
            inflected_word = InflectedWord(
                word=self, value=value, inflection_form=inflection_form)
            store.add(inflected_word)
        else:
            inflected_word.value = value
        return inflected_word



class WordList(Storm):
    """See `IWordList`."""
    zope.interface.implements(IWordList)

    __storm_table__ = "WordList"

    id = Int(primary=True)
    string_id = Unicode()
    name = Unicode()
    language_id = Int(name='language')
    language = Reference(language_id, 'Language.id')
    words = ReferenceSet(
        id,
        'WordWordList.word_list_id',
        'WordWordList.word_id',
        'Word.id')

    def __init__(self, string_id, name, language):
        self.string_id = string_id
        self.name = name
        self.language = language

    def add_word(self, word):
        self.words.add(word)
